function drawRecipIntensityonSphereSurface(qx, qy, qz, DATA, p_type)
% function drawRecipIntensityonSphereSurface(qx, qy, qz, DATA, p_type)
% This function plots intensity on a surface of a sphere of which radius is
% q. Thus q = sqrt(qx^2+qy^2+qz^2) of input DATA is assumed to be amlost identical.
% If not, intensity may be integrated along the q direction for option 3.
% p_type 
% 1 : scatter3 plot
% 2 : polar plot
% 3 : surface plot with integrating DATA for the q dimension.
% type 3 is the fastest plot among these options.

qx = qx(:);
qy = qy(:);
qz = qz(:);
DATA = DATA(:);
switch p_type
    case 1
        figure;
        scatter3(qx, qy, qz, 3, DATA);axis image;
        view([0,0,1])
        box on;
        set(gcf, 'color', 'w')
        xlabel('qx');
        ylabel('qy');
        zlabel('qz');
    case 2
        [th, phi] = cart2sph2(qx, qy, qz);
        %th : latticetude
        %phi : azimuthal angle
        figure;
        polar(phi, th, 'o');
        set(gcf, 'color', 'w')
    case 3
        [th, phi, qn] = cart2sph2(qx, qy, qz);
        qmean = sum(qn.*DATA)/sum(DATA);
        if min(qz) <0
            n_th = (0:1:180)*pi/180;
        else
            n_th = (1:1:90)*pi/180;
        end
        n_phi = (-180:1:180)*pi/180;
        [N1, binth] = histc(th, n_th);
        [N2, binphi] = histc(phi, n_phi);
        N_th = length(N1);
        N_phi = length(N2);
        IND = sub2ind([N_phi, N_th],binphi,binth);
        c=accumarray(IND,DATA);
        if numel(c) < N_th*N_phi;
            c = [c;zeros(N_th*N_phi-numel(c), 1)];
            c = reshape(c, [N_th, N_phi]);
        end
        [o_th, o_phi] = meshgrid(n_th, n_phi);
        [nx, ny, nz] = sph2cart2(o_th(:), o_phi(:), qmean*ones(numel(o_th), 1));
        nx = reshape(nx, size(c));
        ny = reshape(ny, size(c));
        nz = reshape(nz, size(c));
        figure;
        surface(nx, ny, nz, c, 'EdgeColor','none','FaceColor','texturemap');
        axis image
        view([0,0,1])
        set(gcf, 'color', 'w')
        xlabel('qx');
        ylabel('qy');
        zlabel('qz');
end