function [peaks, R] = extractPeaks_respSpaceMap(Qv, DATA, cellinfo, hkls, dq, peakInd)
% function [peaks, R] = extractPeaks_respSpaceMap(Qv, DATA, cellinfo, hkls, dq, peakInd)
% Qv and DATA are the result of construct_RecpSpace.m
% cellinfo requires only recimat, which is B matrix.
% hkls : it requires to have .HKL and .HKLs
% dq : delta q of q0 peaks.
% 
% output:
%   peaks : array indices of Qv and DATA for hkls.
%   R : crystal orientation matrix determined assuming Qv and DATA is from
%   a single crystal
%
% 8/8/2016
% Byeongdu Lee
if nargin < 4
    hkls(1).HKL = [1 1 0];
    hkls(1).HKLs = [1     1     0     1     0    -1    -1    -1     0    -1     0   1;
     1     0     1     0     1     1    -1     0    -1     0    -1  -1;
     0     1     1    -1    -1     0     0    -1    -1     1     1  0];
end
if nargin < 5
    dq = [];
end
if nargin < 6
    peakInd = [];
end
if isempty(peakInd)
    peakInd = 1:length(hkls);
end
B = cellinfo.recimat;
peaks = cell(numel(peakInd), 1);
q = sqrt(Qv(:,1).^2+Qv(:,2).^2+Qv(:,3).^2);
for j=1:numel(peakInd)
    % first order peak (110)
    Np = peakInd(j);
    qvector = hkls(Np).HKL*B';
    q0 = norm(qvector);
    if isempty(dq)
        qindexrange = q0 + [-q0/20, q0/20];
    else
        qindexrange = q0 + [-dq, dq];
    end
    t = q>qindexrange(1) & q < qindexrange(2);
    mv = nanmean(DATA(t));
    mstd = nanstd(DATA(t));
    t = t & DATA > mv+mstd;
    peaks{j} = t;
end

%% Crystal orientation matrix determination.
qx0 = Qv(:,1);
qy0 = Qv(:,2);
qz0 = Qv(:,3);

% Clustering
X = [qx0(peaks{1}), qy0(peaks{1}), qz0(peaks{1})];
[ta, ~, tc] = kmeans(X', 50);
% plotClass(X', ta)

% Determine two orthogonal 110 from the clusters.
edges = unique(ta);
[cnt, neg] = histcounts(ta, edges);
[~, optLabelind] = min(cnt);
optLabel = neg(optLabelind);
v110 = tc.means(:,optLabel)';%v110 = v110/norm(v110);
% finding a normal vector to v110
t = v110*tc.means/norm(v110).^2;
[~, ind] = sort(abs(t));
v2 = tc.means(:, ind(2))'; % 1 -1 0

% Find the rotation matrix 
[Qi, ~] = qr([v110; v2]'); 
% finding hkl that is normal to v110.
v110p = hkls(1).HKL*B';
v2all = B*hkls(1).HKLs;
prodv110v2all = v110p*v2all;
[~, ind] = sort(prodv110v2all);
[Qf, ~] = qr([v110p; v2all(:, ind(2))']'); % 1 1 0; 1 -1 0
R = Qf*inv(Qi);