function [p, beamprofile2, rocking] = gisaxs_rockinganalysis(varargin)
% gisaxs Geometric factor correction
% gisaxs_rockinganalysis(beamprofilescan, rocking, [fullbeam, darkbeam])
% gisaxs_rockinganalysis(beamprofilescan, rocking, [I0beam, fullbeam, darkbeam])
% gisaxs_rockinganalysis(beamprofilescan, rocking, sav after alignment, [fullbeam, darkbeam])
% Beamprofilescan should show a Hiviside function type curve with max fullbeam and
% min darkbeam.
% see also gisaxs_fluxonsample.m
if numel(varargin) == 3
    beamprofilescan = varargin{1};
    rocking = varargin{2};
    beam = varargin{3};
    if numel(beam) == 3
        I0beam = beam(1);
        fullbeam = beam(2);
        darkbeam = beam(3);
    else
        I0beam = max(rocking(:,2));
        fullbeam = beam(1);
        darkbeam = beam(2);
    end
    zs = NaN;
else
    beamprofilescan = varargin{1};
    rocking = varargin{2};
    zs = varargin{3};
    beam = varargin{4};
    fullbeam = beam(1);
    darkbeam = beam(2);
    I0beam = NaN;
end
% convert rocking scan [angle, intensity] into [angle, sav].
[~, k] = min(abs(beamprofilescan(:,2) - mean(beamprofilescan(:,2))));
diff_beam = -diff(beamprofilescan(:,2));
ind = find(diff_beam <= 0);
t = find(ind<k);
if ~isempty(t)
    roi(1) = ind(t(end));
else
    roi(1) = 1;
end
maxB = mean(beamprofilescan(1:roi(1), 2));
t = find(ind>k);
if ~isempty(t)
    roi(2) = ind(t(1));
else
    roi(2) = numel(beamprofilescan(:,2));
end
minB = mean(beamprofilescan(roi(2):end, 2));
beamprofilescan(:,2) = (beamprofilescan(:,2) - minB)/(maxB-minB);
%beamprofilescan(:,2) = (beamprofilescan(:,2) - darkbeam)/(fullbeam-darkbeam);

%diff_beam = -diff(beamprofilescan(:,2));
%[~, peakp] = max(diff_beam);
%neg_slope = find(diff_beam <= 0);    
%t = find(neg_slope < peakp);roi(1) = neg_slope(t(end));
%t = find(neg_slope > peakp);
%if isempty(t)
%    roi(2) = numel(beamprofilescan(:,2));
%else
%    roi(2) = neg_slope(t(1));
%end
beamprofile2 = beamprofilescan(roi(1):roi(2), :);

if isnan(zs) & ~isnan(I0beam)
    I0beam = (I0beam - darkbeam)/(fullbeam-darkbeam);
    zs = interp1(beamprofile2(:,2), beamprofile2(:,1), I0beam); % sample height
    fprintf('Sample is located at %0.3f, which is at %0.3f fractional height\n', zs, I0beam)
end
%beamprofile2(:,1) = beamprofile2(:,1) - zs;
%zs = 0;

% massage.... rocking scan.
mxv = max(rocking(:,2));
t = rocking(:,2) < mxv*0.5;
rocking(t, :) = [];

rocking(:,2) = (rocking(:,2) - darkbeam)/(fullbeam-darkbeam);
zpos = interp1(beamprofile2(:,2), beamprofile2(:,1), rocking(:,2));
rocking = [rocking(:,1), zpos];
%[~, indm] = max(zpos);
%up = rocking(indm:end, :);
%down = rocking(1:indm, :);
%up_pos = interp1(beamprofile2(:,2), beamprofile2(:,1), up(:,2));
%down_pos = interp1(beamprofile2(:,2), beamprofile2(:,1), down(:,2));
%f1 = ezfit(up(:,1), up_pos, sprintf('y = Lu*sin((x+x0)*pi/180)+z0; z0=%f', zs));
%f2 = ezfit(down(:,1), down_pos, sprintf('y = -1*Ld*sin((x+x0)*pi/180)+z0; z0=%f', zs));

%cv = FitLee_fitfunction(p, y, fitFunHandle, q, pnames, tag)
fitFunHandle = @FitLee_rocking;
p = fitFunHandle();
options = optimset('fminsearch');
options = optimset(options, 'TolX',0.1E-8);
%options = optimset(options, 'PlotFcns',@optimplotx);
%options = optimset(options, 'OutputFcn',@BLFit_outfun);
options = optimset(options, 'MaxIter',50000);
options = optimset(options, 'MaxFunEvals', 50000);
inip = cell2mat(struct2cell(p));
inip(end) = zs;
pnames = fieldnames(p);
bestP = fminsearchcon(@(x) FitLee_fitfunction(x, rocking(:,2), fitFunHandle, rocking(:,1), pnames),...
    inip,[-0.5, 0.1, 0.1, beamprofilescan(1,1)],[0.5,50, 50, beamprofilescan(end,1)], [], [], [], options);
p = cell2struct(num2cell(bestP), pnames, 1);
%p.zs = zs;