function ret = gisaxsgeofactorcorrection(rocking, I_0_angle, L, background, fullbeam)
% gisaxs Geometric factor correction
% input:
% rocking = [aiIR, IR]
%   aiIR : incident angle axis
%   IR : Rocking intensity axis
% L : sample length
% I_0_angle : angle where Rocking intensity becomes 0 (deg)
% background : flat background in the rocking scan.
% fullbeam: full beam intensity measured for the same exposuretime.
if nargin < 4
    background = 0;
end
IR = rocking(:,2) - background;
aiIR = rocking(:,1);
if nargin < 5
    fullbeam = max(IR)*2;
end

%I_0_angle = 0.1666; % angle where Rocking intensity becomes 0 (deg)
%L = 5.5; % sample length (mm)
b = L*sin(deg2rad(I_0_angle)); % beam size.
disp(sprintf('Beam size was %0.3f mm', b))

Cg = (1-2*IR/fullbeam)*L;
t = find(aiIR>I_0_angle);
Cg(t) = b./sin(deg2rad(aiIR(t)));
ret = [aiIR(:), Cg(:)];