function fluxonsample = gisaxsgeofactorcorrection2(varargin)
% gisaxs Geometric factor correction
% gisaxs_rockinganalysis(beamprofilescan, rocking, [I0beam, fullbeam, darkbeam])
% gisaxs_rockinganalysis(beamprofilescan, rocking, sav after alignment, [fullbeam, darkbeam])
% Beamprofilescan should show a Hiviside function type curve with max fullbeam and
% min darkbeam.
%
if numel(varargin) == 3
    beamprofilescan = varargin{1};
    rocking = varargin{2};
    beam = varargin{3};
    I0beam = beam(1);
    fullbeam = beam(2);
    darkbeam = beam(3);
    zs = NaN;
else
    beamprofilescan = varargin{1};
    rocking = varargin{2};
    zs = varargin{3};
    beam = varargin{4};
    fullbeam = beam(1);
    darkbeam = beam(2);
    I0beam = NaN;
end
% convert rocking scan [angle, intensity] into [angle, sav].
beamprofilescan(:,2) = (beamprofilescan(:,2) - darkbeam)/(fullbeam-darkbeam);
diff_beam = -diff(beamprofilescan(:,2));
[~, peakp] = max(diff_beam);
neg_slope = find(diff_beam <= 0);    
t = find(neg_slope < peakp);roi(1) = neg_slope(t(end));
t = find(neg_slope > peakp);
if isempty(t)
    roi(2) = numel(beamprofilescan(:,2));
else
    roi(2) = neg_slope(t(1));
end
beamprofile2 = beamprofilescan(roi(1):roi(2), :);

if ~isnan(I0beam)
    I0beam = (I0beam - darkbeam)/(fullbeam-darkbeam);
    zs = interp1(beamprofile2(:,2), beamprofile2(:,1), I0beam); % sample height
    fprintf('Sample is located at %0.3f, which is at %0.3f fractional height\n', zs, I0beam)
end

% massage.... rocking scan.
rocking(:,2) = (rocking(:,2) - darkbeam)/(fullbeam-darkbeam);
zpos = interp1(beamprofile2(:,2), beamprofile2(:,1), rocking(:,2))-zs;
rocking = [rocking(:,1), zpos];
%[~, indm] = max(zpos);
%up = rocking(indm:end, :);
%down = rocking(1:indm, :);
%up_pos = interp1(beamprofile2(:,2), beamprofile2(:,1), up(:,2));
%down_pos = interp1(beamprofile2(:,2), beamprofile2(:,1), down(:,2));
%f1 = ezfit(up(:,1), up_pos, sprintf('y = Lu*sin((x+x0)*pi/180)+z0; z0=%f', zs));
%f2 = ezfit(down(:,1), down_pos, sprintf('y = -1*Ld*sin((x+x0)*pi/180)+z0; z0=%f', zs));

%cv = FitLee_fitfunction(p, y, fitFunHandle, q, pnames, tag)
fitFunHandle = @FitLee_rocking;
p = fitFunHandle();
options = optimset('fminsearch');
options = optimset(options, 'TolX',0.1E-5);
options = optimset(options, 'PlotFcns',@optimplotx);
options = optimset(options, 'OutputFcn',@BLFit_outfun);
options = optimset(options, 'MaxIter',5000);
options = optimset(options, 'MaxFunEvals', 5000);
inip = struct2cell(p);
pnames = fieldnames(p);
bestP = fminsearchcon(@(x) FitLee_fitfunction(x, rocking(:,2), fitFunHandle, rocking(:,1), pnames),...
    inip,[-0.5, 0.1, 0.1],[0.5,50, 50], [], [], [], options);
p = cell2struct(num2cell(bestP), pnames, 1);


ang = linspace(p.a0, 1, 1000);
sav_pos_ai = fitFunHandle(p, ang);
Ibeam = (Ibeam - darkbeam)/(fullbeam-darkbeam);
z_up = interp1(beamprofile2(:,2), beamprofile2(:,1), Ibeam);
ai = interp1(sav_pos_ai, ang, z_up-zs)-p.a0;
z_down = z_up - (p.Lu+p.Ld)*sin(ai*pi/180);
fluxonsample = interp1(beamprofile2(:,1), beamprofile2(:,2), z_down) - ...
    interp1(beamprofile2(:,1), beamprofile2(:,2), z_up);