function sresult = SAXSLee(varargin)
% SAXS Program to read spec files
% 
% Copyright 2009 Byeongdu Lee

hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
if isempty(hFigSAXSLee)
    initFigure;
else
    figure(hFigSAXSLee);
    sresult = hFigSAXSLee;
    return;
end

% --- turn off waring;
warning off MATLAB:Axes:NegativeDataInLogAxis;


%================================================================
% --- initialize figure layout
%================================================================
function initFigure
% --------------------------------
% --- main figure handle
% --------------------------------
posScreen   = get(0,'screenSize');
hFigWidth   = 680;
hFigHeight  = 531;
hFigPos     = [...
    posScreen(3)/2-hFigWidth/2,...
    posScreen(4)/2-hFigHeight/2,...
    hFigWidth,hFigHeight];
hFigSAXSLee = figure(...
    'BackingStore','on',...
    'Units','pixels',...
    'DockControls','off',...
    'Resize','on',...
    'ResizeFcn',@SAXSLee_ResizeFcn,...
    'PaperOrient','portrait',...
    'PaperPositionMode','auto',...
    'IntegerHandle','off',...
    'NumberTitle','off',...
    'MenuBar','none',...
    'Toolbar','none',...
    'Name','Spec Reader',...
    'Position',hFigPos,...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_Fig',...
    'CreateFcn',@SAXSLee_CreateFcn,...
    'CloseRequestFcn',@SAXSLee_CloseRequestFcn,...
    'UserData',[]);
SAXSLee_datalabel on
% --------------------------------
% --- axes handle
% --------------------------------
hAxes = axes(...
    'Parent',hFigSAXSLee,...
    'Box','on',...
    'XGrid','on',...
    'YGrid','on',...
    'Tag','SAXSLee_Axes');

% --------------------------------
% --- file menu handles
% --------------------------------
hMenuFile = uimenu(hFigSAXSLee,...
    'Label','&File',...
    'Position',1,...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFile');
hMenuFileOpenCurrent = uimenu(hMenuFile,...
    'Label','&Open Current Spec File...',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFileOpenCurrent',...
    'Accelerator','O',...
    'callback',{@SAXSLee_openspec, 0});
hMenuFileOpen = uimenu(hMenuFile,...
    'Label','&Open Spec File...',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFileOpen',...
    'Accelerator','O',...
    'callback',@SAXSLee_openspec);
hMenuFileClose = uimenu(hMenuFile,...
    'Label','&Close',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFileClose',...
    'Accelerator','W',...
    'callback',@SAXSLee_CloseRequestFcn);
hMenuFileSave = uimenu(hMenuFile,...
    'Label','&Save',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFileSave',...
    'Accelerator','S',...
    'callback','curvesave');
hMenuFileSave2Workspace = uimenu(hMenuFile,...
    'Label','Save to &Workspace',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFileSave2Workspace',...
    'callback','save2workspace');
hMenuFileClear = uimenu(hMenuFile,...
    'Label','&Clear Current Selection',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFileClear',...
    'callback','SAXSLee_fileclear');
hMenuFilePreferences = uimenu(hMenuFile,...
    'Label','Pre&ferences...',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFilePreferences',...
    'callback','preferences');
hMenuFilePageSetup = uimenu(hMenuFile,...
    'Label','Pa&ge Setup...',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFilePageSetup',...
    'callback','pagesetupdlg');
hMenuFilePrintPreview = uimenu(hMenuFile,...
    'Label','Print Pre&view...',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFilePrintPreview',...
    'callback',@SAXSLee_PrintPreviewFcn);
hMenuFilePrint = uimenu(hMenuFile,...
    'Label','&Print...',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFilePrint',...
    'Accelerator','P',...
    'callback',@SAXSLee_PrintFcn);
hMenuFilePrintToFigure = uimenu(hMenuFile,...
    'Label','Print to Figure...',...
    'Accelerator','G',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuFilePrintToFigure',...
    'callback','print2figure');

% --------------------------------
% --- tools menu handles
% --------------------------------
hMenuTools = uimenu(hFigSAXSLee,...
    'Label','&Tools',...
    'Position',2,...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuTools');
hMenuToolsShowScan = uimenu(hMenuTools,...
    'Label','Sho&w Current Scan',...
    'Callback','showscan',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuToolsShowScan');
hMenuToolsShowScan = uimenu(hMenuTools,...
    'Label','&Load scans from directories',...
    'Callback','SAXSLee_loadDir',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuToolsLoadDir');
hMenuToolsInvertX = uimenu(hMenuTools,...
    'Label','&Select and Set reference',...
    'Callback',@SAXSLee_selectref,...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuToolsInvertX');
hMenuToolsFoldX = uimenu(hMenuTools,...
    'Label','Fold X &Axis (New Figure) ...',...%    'Callback','foldxaxis',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuToolsFoldX');
hMenuToolsSubBKG = uimenu(hMenuTools,...
    'Label','Subtract Constant &Background',...
    'Callback','subtractbkg',...
    'separator','on',...
    'HandleVisibility','callback',...
    'Accelerator','B',...    
    'Tag','SAXSLee_MenuToolsISubBKG');
hMenuToolsScanMerge = uimenu(hMenuTools,...
    'Label','&Merge Scans',...
    'HandleVisibility','callback',...
    'Accelerator','M',...
    'Callback','SAXSLee_scanmerge');
hMenuToolsConvert = uimenu(hMenuTools,...
    'Label','&Auto Background subtraction',...
    'Callback','convert2theta2qz',...
    'HandleVisibility','callback',...
    'Accelerator','T',...
    'Tag','SAXSLee_MenuToolsConvert');
hMenuToolsFootprint = uimenu(hMenuTools,...
    'Label','&reset reference scans',...
    'Callback','resetrefscan',...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuToolsFootprint');
hMenuToolsNormalize = uimenu(hMenuTools,...
    'Label','&Normalize...',...
    'HandleVisibility','callback',...
    'Callback','SAXSLee_normalize');
hMenuToolsXNormalize = uimenu(hMenuTools,...
    'Label','&XAxis Normalize...',...
    'HandleVisibility','callback',...
    'Callback','SAXSLee_Xnormalize');
hMenuToolsAnalysis0 = uimenu(hMenuTools,...
    'Label','&Guinier and Porod analysis ...',...
    'HandleVisibility','callback',...
    'Callback','SAXSLee_analysis');
hMenuToolsRocking = uimenu(hMenuTools,...
    'Label','&fitdata ...',...
    'HandleVisibility','callback',...
    'Callback','fitsaxslee');
hMenuToolsEngwav = uimenu(hMenuTools,...
    'Label','Calculate &Energy/Wavelength...',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Callback','engwav');
hMenuToolsTrueRef = uimenu(hMenuTools,...
    'Label','&True Reflectivity...',...
    'HandleVisibility','callback',...
    'Callback','trueref');
hMenuToolsSettings = uimenu(hMenuTools,...
    'Label','&Settings...',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Callback','SAXSLeesetting');
    
% --------------------------------
% --- help menu handles
% --------------------------------
hMenuHelp = uimenu(hFigSAXSLee,...
    'Label','&Help',...
    'Position',3,...
    'HandleVisibility','callback',...
    'Tag','SAXSLee_MenuHelp');    
hMenuHelpSAXSLee = uimenu(hMenuHelp,...
    'Label','&Spec Reader Help',...
    'HandleVisibility','callback',...
    'Callback',@SAXSLee_Help);
hMenuHelpAbout = uimenu(hMenuHelp,...
    'Label','&About Spec Reader',...
    'HandleVisibility','callback',...
    'Separator','on',...
    'Callback',@SAXSLee_AboutSAXSLee);
    
% --------------------------------
% --- toolbar handles
% --------------------------------
icons = load('SG_icons.mat');
try
    icon_minus = imread('minus.png');
catch
    mn = fullfile(matlabroot,'toolbox','mytool','1DSAXS', 'GUI', 'minus.png');
    icon_minus = imread(mn);
end
t = find(icon_minus == 0);icon_minus=double(icon_minus)/255;
icon_minus(t)= NaN;

try
    icon_arrow = imread('arrow.png');
catch
    mn = fullfile(matlabroot,'toolbox','mytool','1DSAXS', 'GUI', 'arrow.png');
    icon_arrow = imread(mn);
   
end
t = find(icon_arrow == 0);icon_arrow=double(icon_arrow)/255;
icon_arrow(t)= NaN;

try
    icon_noentry = imread('noentry-circlesign.png');
catch
    mn = fullfile(matlabroot,'toolbox','mytool','1DSAXS', 'GUI', 'noentry-circlesign.png');
    icon_noentry = imread(mn);
end
t = find(icon_noentry == 0);icon_noentry=double(icon_noentry)/255;
icon_noentry(t)= NaN;

try
    icon_tick = imread('tick.png');
catch
    mn = fullfile(matlabroot,'toolbox','mytool','1DSAXS', 'GUI', 'tick.png');
    icon_tick = imread(mn);
end
t = find(icon_tick == 0);icon_tick=double(icon_tick)/255;
icon_tick(t)= NaN;

try
    icon_image = imread('HDF_rasterimage.gif');
catch
    mn = fullfile(matlabroot, 'toolbox', 'matlab', 'icons', 'HDF_rasterimage.gif');
    icon_image = imread(mn);
end
icon_image = icon_image/36*255;
icon_image = ind2rgb(icon_image, jet(128));
%t = find(icon_image == 0);icon_image=double(icon_image)/36;
%icon_image(t)= NaN;

hToolbar = uitoolbar(hFigSAXSLee,...
    'Tag','SAXSLee_Toolbar');
hToolbarOpen = uipushtool(hToolbar,...
    load(fullfile(matlabroot,'toolbox','matlab','icons','opendoc.mat')),...
    'TooltipString','Open Spec File',...
    'ClickedCallback',@SAXSLee_openspec,...
    'Tag','toolbarOpen');
hToolbarSave = uipushtool(hToolbar,...
    load(fullfile(matlabroot,'toolbox','matlab','icons','savedoc.mat')),...
    'TooltipString','Save Data',...
    'ClickedCallback','curvesave',...
    'Tag','toolbarSave');
hToolbarPrint = uipushtool(hToolbar,...
    load(fullfile(matlabroot,'toolbox','matlab','icons','printdoc.mat')),...
    'TooltipString','Print Figure',...
    'ClickedCallback',@SAXSLee_PrintFcn,...
    'Tag','toolbarPrint');
hToolbarEditPlot = uitoggletool(hToolbar,...
    load(fullfile(matlabroot,'toolbox','matlab','icons','pointer.mat')),...
    'TooltipString','Edit Plot',...
    'ClickedCallback',@toolbarEditPlotFcn,...
    'Separator','on',...
    'Tag','toolbarEditPlot');
iconToolbarZoom = load(...
    fullfile(matlabroot,'toolbox','matlab','icons','zoom.mat'));
hToolbarZoom = uitoggletool(hToolbar,...
    'CDATA',iconToolbarZoom.zoomCData,...
    'TooltipString','Zoom',...
    'Separator','on',...
    'State','off',...
    'ClickedCallback',@toolbarZoomFcn,...
    'Tag','toolbarZoom');
hToolbarPan = uitoggletool(hToolbar,...
    load(fullfile(matlabroot,'toolbox','matlab','icons','pan.mat')),...
    'TooltipString','Pan',...
    'ClickedCallback',@toolbarPanFcn,...
    'Tag','toolbarPan');
hToolbarDataCursor = uitoggletool(hToolbar, ...
    load(fullfile(matlabroot,'toolbox','matlab','icons','datatip.mat')),...
    'TooltipString','Data Cursor',...
    'ClickedCallback',@toolbarDataCursorFcn,...
    'Separator','on',...
    'Tag','toolbarDataCursor');
hToolbarLegend = uitoggletool(hToolbar, ...
    load(fullfile(matlabroot,'toolbox','matlab','icons','legend.mat')),...
    'TooltipString','Legend On/Off',...
    'ClickedCallback',@toolbarLegendFcn,...
    'Separator','on',...
    'Tag','toolbarLegend');
hToolbarGrid = uitoggletool(hToolbar,...
    'CData',icons.icongrid,...
    'TooltipString','Grid On/Off',...
    'ClickedCallback','grid',...
    'State','on',...
    'Tag','toolbarGrid');
iconPlottoolsOff = load(...
    fullfile(matlabroot,'toolbox','matlab','icons','plottoolsoff.mat'));
hToolbarPlottoolsOff = uipushtool(hToolbar,...
    'CData',iconPlottoolsOff.cdata,...
    'TooltipString','Hide Plot Tools',...
    'ClickedCallback',@toolbarPlottoolsOffFcn,...
    'Separator','on',...
    'Enable','off',...
    'Tag','toolbarPlottoolsOff');
iconPlottoolsOn = load(...
    fullfile(matlabroot,'toolbox','matlab','icons','plottoolson.mat'));
hToolbarPlottoolsOn = uitoggletool(hToolbar,...
    'CData',iconPlottoolsOn.cdata,...
    'TooltipString','Hold on',...
    'ClickedCallback',@toolbarHoldOnFcn,...
    'Enable','on',...
    'Tag','toolbarPlottoolsOn');
hToolbarAutoImageOn = uitoggletool(hToolbar,...
    'CData',icon_image,...%iconPlottoolsOn.cdata,...
    'TooltipString','Auto image on',...
    'Enable','on',...
    'Tag','hToolbarAutoImageOn');
%hToolbarPlottoolsOn = uipushtool(hToolbar,...
%    'CData',iconPlottoolsOn.cdata,...
%    'TooltipString','Show plot on',...
%    'ClickedCallback',@toolbarPlottoolsOnFcn,...
%    'Enable','on',...
%    'Tag','toolbarPlottoolsOn');
hToolbarMonitorAvg = uitoggletool(hToolbar,...
    'CData',icons.iconmonitor.play,...
    'TooltipString','Auto azimuthal averaging On/Off',...
    'Separator','on',...
    'ClickedCallback','SAXSLee_autoazimuthalaverage',...
    'State','off',...
    'Tag','toolbarMonitorAvg');
hToolbarMonitor = uitoggletool(hToolbar,...
    'CData',icons.iconmonitor.play,...
    'TooltipString','Scan Monitor On/Off',...
    'ClickedCallback','SAXSLee_monitor',...
    'State','off',...
    'Tag','toolbarMonitor');
hToolbarInvert = uipushtool(hToolbar,...
    'CData',icon_tick,...%icons.iconinvertx,...
    'Separator','on',...    
    'TooltipString','Set the selected as background for subtraction',...
    'ClickedCallback',@SAXSLee_selectref,...
    'Tag','toolbarInvert');
hToolbarRefScan = uitoggletool(hToolbar,...
    'CData',icons.iconfoldx,...
    'TooltipString','Show background scan',...
    'ClickedCallback',@refscanplot,...
    'Tag','toolbarRefScan');
hToolbarMerge = uipushtool(hToolbar,...
    'CData',icons.iconmerge,...
    'Separator','on',...  
    'TooltipString','Merge Scans',...
    'ClickedCallback','SAXSLee_scanmerge',...
    'Tag','toolbarMerge');
hToolbarAutobacksub = uitoggletool(hToolbar,...
    'CData',icon_minus,...
    'TooltipString','Auto Background Subtraction',...
    'ClickedCallback','autobacksub',...
    'Tag','toolbarAutobacksub');
%hToolbarAutobacksub = uitoggletool(hToolbar,...
%    'CData',icons.iconconvert,...
%    'TooltipString','Auto Background Subtraction',...
%    'ClickedCallback','autobacksub',...
%    'Tag','toolbarAutobacksub');
hToolbarFootprint = uipushtool(hToolbar,...
    'CData',icon_noentry,...%icons.iconfootprint,...
    'TooltipString','Reset reference scans',...
    'ClickedCallback','resetrefscan',...
    'Tag','toolbarFootprint');
hToolbarFitsaxslee = uipushtool(hToolbar,...
    'CData',icon_arrow,...%icons.iconrocking,...
    'TooltipString','fit data in the selected region',...
    'ClickedCallback','fitsaxslee',...
    'Tag','toolbarFitsaxslee');
hToolbarSettings = uipushtool(hToolbar,...
    'CData',icons.iconsettings,...
    'Separator','on',...    
    'TooltipString','Settings',...
    'ClickedCallback','SAXSLeesetting',...
    'Tag','toolbarSettings');
hToolbarReadValue = uitoggletool(hToolbar,...
    load(fullfile(matlabroot,'toolbox','matlab','icons','datatip.mat')),...
    'TooltipString','Read Value',...
    'ClickedCallback',@gtrack_start,...
    'Separator','on',...
    'Tag','Read value');

% --------------------------------
% --- pushbotton handles
% --------------------------------
hPushbuttonSelectScan = uicontrol(hFigSAXSLee,...
    'Style','pushbutton',...
    'String','Select Scan',...
    'TooltipString',' Select Scan',...
    'Tag','SAXSLee_PushbuttonSelectScan',...
    'callback',@SAXSLee_selectscan);
hPushbuttonRefresh = uicontrol(hFigSAXSLee,...
    'Style','pushbutton',...
    'String','Load 1d data',...
    'TooltipString',' Replot',...
    'Tag','SAXSLee_PushbuttonRefresh',...
    'callback','SAXSLee_reflist2');
%hPushbuttonRefresh = uicontrol(hFigSAXSLee,...
%    'Style','pushbutton',...
%    'String','Load 1d data',...
%    'TooltipString',' Replot',...
%    'Tag','SAXSLee_PushbuttonRefresh',...
%    'callback','refreshscanplot');

% --------------------------------
% --- popupmenu handles
% --------------------------------
hPopupmenuX = uicontrol(hFigSAXSLee,...
    'Style','popupmenu',...
    'Background','w',...
    'String',' ',...
    'TooltipString',' Select Spec file',...
    'Tag','SAXSLee_PopupmenuX',...
    'callback',@SAXSLee_switchspecfile);
hPopupmenuY = uicontrol(hFigSAXSLee,...
    'Style','popupmenu',...
    'Background','w',...
    'String',' ',...
    'TooltipString',' Select Background from Reference Scans',...
    'Tag','SAXSLee_PopupmenuY',...
    'callback',@SAXSLee_highlight);
hPopupmenuI0 = uicontrol(hFigSAXSLee,...
    'Style','edit',...
    'Background','w',...
    'String','1',...
    'TooltipString',' transmittance for background',...
    'Tag','SAXSLee_PopupmenuI0');
%    'callback',@scanplot);
hPopupmenuPlotStyle = uicontrol(hFigSAXSLee,...
    'Style','popupmenu',...
    'Background','w',...
    'String',{'linear','logx','logy','logxy'},...
    'TooltipString',' Plot Style',...
    'Tag','SAXSLee_PopupmenuPlotStyle',...
    'callback',@plotstyle);


%================================================================
% --- things to do when starting SAXSLee:
% 1. initalize the settings; save to application data 'settings'
%================================================================
function SAXSLee_CreateFcn(hObject,eventdata)
hFigSAXSLee = hObject;
settings.wavelength = 1.024755735383;
settings.footprintAngle     = 0.5;              % incident side footprint angle
settings.footprintAngleSC   = 0.5;              % detector side footprint angle
settings.georockingSCFlag   = 0;                % 0) no detector side correction 1) yes
settings.qz4Rocking         = 0.15;             % default qz for rocking scan
settings.merge.mode  = 2;        % 1) intensity based 2) # of points based (default) 
settings.merge.interpMethod = 1; % 1) spline (default) 2) linear 3) nearest 4) cubic
settings.monitorPeriod   = 0.5;  % scan monitor period (sec)
settings.average.mode = 2;       % 1) goldaverage (default) 2) SAXSimageviewer
settings.automethod = 1;       % 1) specfile (default) 2) directory comparison
settings.file       = '';
settings.scan       = '';
settings.back       = '';
settings.rootDir    = '';
settings.path       = '';
settings.averageDir       = 'Averaged';
settings.averageCmd       = './goldnormengavg';
settings.backsubtractedDir       = 'BackSub';
settings.imageListString = '*.tif';
settings.savepath   = '';
settings.trueref.openfiles = {};
settings.trueref.openpath = '';
setappdata(hFigSAXSLee,'settings',settings);
setappdata(hFigSAXSLee,'refscan',[]);
%assignin('base', 'SAXSLee', hFigSAXSLee);
fprintf(' Use the following command to access settings\n')
fprintf('       getappdata(SAXSLee, "setting")\n')

%================================================================
% --- SAXSLee figure close request function
%================================================================
function SAXSLee_CloseRequestFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hFigShowScan = findall(0,'Tag','figSAXSLeeShowScan');
hFigTrueref = findall(0,'Tag','trueref_Fig');
hPopupmenuX = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuX');
hPopupmenuY = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuY');
hPopupmenuI0 = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuI0');
hPopupmenuPlotStyle = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuPlotStyle');
hAxes = findall(hFigSAXSLee,'Tag','SAXSLee_Axes');
hSAXSLeetimerMonitor   = timerfindall('Tag','SAXSLeetimerMonitor');
settings = getappdata(hFigSAXSLee,'settings');
file = settings.file;
scan = settings.scan;
% --- if no plot, then close without request
if ~isempty(file) & ~isempty(scan) & isfield(scan,'selection') & ~isempty(scan.selection{1})
    quitButton = questdlg('Quit without saving data ?',...
        'Spec Reader - Confirm Close','Quit','Cancel','Cancel');
    switch quitButton
        case 'Cancel'
            % take no action
            return;
        case 'Quit'
    end
end
%resetfigSAXSLee;
warning on MATLAB:Axes:NegativeDataInLogAxis;
if ~isempty(hFigTrueref)
    delete(hFigTrueref);
end
if ~isempty(hFigShowScan)
    delete(hFigShowScan);
end
delete(hFigSAXSLee);
return;


%================================================================
% --- SAXSLee figure resize function
%================================================================
function SAXSLee_ResizeFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hAxes = findall(hFigSAXSLee,'Tag','SAXSLee_Axes');
hPushbuttonSelectScan = findall(hFigSAXSLee,'Tag','SAXSLee_PushbuttonSelectScan');
hPushbuttonRefresh = findall(hFigSAXSLee,'Tag','SAXSLee_PushbuttonRefresh');
hPopupmenuX = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuX');
hPopupmenuY = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuY');
hPopupmenuI0 = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuI0');

hPopupmenuPlotStyle = findall(hFigSAXSLee,'Tag','SAXSLee_PopupmenuPlotStyle');
figureSize = get(hFigSAXSLee,'Position');
PosFigureTopCenter = [(figureSize(3))/2,figureSize(4)-35];
try         % in case figure size is too samll that width and height < 0
    PosFigureTopCenter(1) = PosFigureTopCenter(1) - 30;
    set(hPushbuttonSelectScan,...
        'Units','Pixels',...
        'Position',[PosFigureTopCenter(1)-220,PosFigureTopCenter(2),80,25]);
    set(hPushbuttonRefresh,...
        'Units','Pixels',...
        'Position',[PosFigureTopCenter(1)+230,PosFigureTopCenter(2),80,25]);
    set(hPopupmenuX,...
        'Units','Pixels',...
        'Position',[PosFigureTopCenter(1)-130,PosFigureTopCenter(2),80,25]);
    set(hPopupmenuY,...
        'Units','Pixels',...
        'Position',[PosFigureTopCenter(1)-40,PosFigureTopCenter(2),80,25]);
    set(hPopupmenuI0,...
        'Units','Pixels',...
        'Position',[PosFigureTopCenter(1)+40,PosFigureTopCenter(2),80,25]);
    set(hPopupmenuPlotStyle,...
        'Units','Pixels',...
        'Position',[PosFigureTopCenter(1)+130,PosFigureTopCenter(2),80,25]);
    set(hAxes,...
        'Units','Pixels',...
        'Position',[65,80,figureSize(3)-130,figureSize(4)-160]);
catch
end

%================================================================
% --- print preview callback function
%================================================================
function SAXSLee_PrintPreviewFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hHide = [findall(hFigSAXSLee,'Style','pushbutton');...
         findall(hFigSAXSLee,'Style','popupmenu')];
%set(hHide,'Visible','off');
hPrintPreview = printpreview(hFigSAXSLee);
%set(hHide,'Visible','on');


%================================================================
% --- print callback function
%================================================================
function SAXSLee_PrintFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hHide = [findall(hFigSAXSLee,'Style','pushbutton');...
         findall(hFigSAXSLee,'Style','popupmenu')];
set(hHide,'Visible','off');
printdlg;
set(hHide,'Visible','on');


%================================================================
% --- toolbar plottools off callback function
%================================================================
function toolbarPlottoolsOffFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hToolbarPlottoolsOff = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOff');
hToolbarPlottoolsOn  = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOn');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
if strcmp(get(hToolbarPlottoolsOff,'Enable'),'off')
    return;
else
    zoom off;
    pan off;
    datacursormode off;
    set(hToolbarZoom,'State','off');
    set(hToolbarPan,'State','off');
    set(hToolbarDataCursor,'State','off');
    set(hToolbarPlottoolsOff,'Enable','off');
    set(hToolbarPlottoolsOn,'Enable','on');
    plottools off;
end


%================================================================
% --- toobar plottools on callback function
%================================================================
function toolbarHoldOnFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hAxesSAXSLee = findall(0,'Tag','SAXSLee_Axes');

hToolbarPlottoolsOff = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOff');
hToolbarPlottoolsOn  = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOn');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
if strcmp(get(hToolbarPlottoolsOn,'Enable'),'off')
    return;
else
    if strcmp(get(hObject, 'State'), 'on')
        set(hAxesSAXSLee, 'Nextplot', 'add');
    else
        set(hAxesSAXSLee, 'Nextplot', 'replace');
    end
    %get(hAxesSAXSLee, 'Nextplot')
    %plotedit;
    %zoom off;
    %pan off;
    %datacursormode off;
    %set(hToolbarEditPlot,'State','on');
    %set(hToolbarZoom,'State','off');
    %set(hToolbarPan,'State','off');
    %set(hToolbarDataCursor,'State','off');
    %set(hToolbarPlottoolsOn,'Enable','off');
    %set(hToolbarPlottoolsOff,'Enable','on');
    %plottools on;
end
%================================================================
% --- toobar plottools on callback function
%================================================================
function toolbarImageshow(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hAxesSAXSLee = findall(0,'Tag','SAXSLee_Axes');

hToolbarPlottoolsOff = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOff');
hToolbarPlottoolsOn  = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOn');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
if strcmp(get(hToolbarPlottoolsOn,'Enable'),'off')
    return;
else
    if strcmp(get(hObject, 'State'), 'on')
        set(hAxesSAXSLee, 'Nextplot', 'add');
    else
        set(hAxesSAXSLee, 'Nextplot', 'replace');
    end
end


%================================================================
% --- toobar plottools on callback function
%================================================================
function toolbarPlottoolsOnFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hToolbarPlottoolsOff = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOff');
hToolbarPlottoolsOn  = findall(hFigSAXSLee,'Tag','toolbarPlottoolsOn');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
if strcmp(get(hToolbarPlottoolsOn,'Enable'),'off')
    return;
else
    plotedit;
    zoom off;
    pan off;
    datacursormode off;
    set(hToolbarEditPlot,'State','on');
    set(hToolbarZoom,'State','off');
    set(hToolbarPan,'State','off');
    set(hToolbarDataCursor,'State','off');
    set(hToolbarPlottoolsOn,'Enable','off');
    set(hToolbarPlottoolsOff,'Enable','on');
    plottools on;
end


%================================================================
% --- toolbar edit plot callback
%================================================================
function toolbarEditPlotFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
plotedit;
zoom off;
pan off;
datacursormode off;
set(hToolbarZoom,'state','off');
set(hToolbarPan,'state','off');
set(hToolbarDataCursor,'state','off');

%================================================================
% --- toolbar zoom callback
%================================================================
function toolbarZoomFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
plotedit off;
zoom;
pan off;
datacursormode off;
set(hToolbarEditPlot,'state','off');
set(hToolbarPan,'state','off');
set(hToolbarDataCursor,'state','off');

%================================================================
% --- toolbar pan callback
%================================================================
function toolbarPanFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
plotedit off;
zoom off;
pan;
datacursormode off;
set(hToolbarEditPlot,'state','off');
set(hToolbarZoom,'state','off');
set(hToolbarDataCursor,'state','off');


%================================================================
% --- toolbar datacuror callback
%================================================================
function toolbarDataCursorFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
hToolbarEditPlot = findall(hFigSAXSLee,'Tag','toolbarEditPlot');
hToolbarZoom = findall(hFigSAXSLee,'Tag','toolbarZoom');
hToolbarPan = findall(hFigSAXSLee,'Tag','toolbarPan');
hToolbarDataCursor = findall(hFigSAXSLee,'Tag','toolbarDataCursor');
plotedit off;
zoom off;
pan off;
try
    datacursormode;
catch
end
set(hToolbarEditPlot,'state','off');
set(hToolbarZoom,'state','off');
set(hToolbarPan,'state','off');


%================================================================
% --- toolbar legend callback funcion
%================================================================
function toolbarLegendFcn(hObject,eventdata)
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
curvelegend(hFigSAXSLee);


%================================================================
% --- menu SAXSLee help callback fcn
%================================================================
function SAXSLee_Help(hObject,eventdata)
%helpWeb = fullfile(matlabroot,'toolbox','xraylabtool','help','xraylabtool_page.html');
%helpview(helpWeb);
msgbox('No help!','Help on Spec Reader','Help','modal');
 

%================================================================
% --- menu aboutSAXSLee function callback
%================================================================
function SAXSLee_AboutSAXSLee(hObject,eventdata)
text_string = {...
    'Spec Reader 3.5 (2006-06-15)';...
    ' ';...
    'X-Ray and Soft Condensed Matter Group';...
    'Department of Physics';...
    'University of California, San Diego';...
    'Copyright 2004-2006 Zhang Jiang';...
    'Email: zjiang@physics.ucsd.edu'};
icons = load('SG_icons.mat');
msgbox(text_string,'About Spec Reader','custom',icons.logo.cdata,icons.logo.colormap,'modal');



function gtrack_OnMouseMove(src,evnt)

% get mouse position
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
    hAxes = findall(hFigSAXSLee,'Tag','SAXSLee_Axes');
    pos_hAxes = get(hAxes, 'position');
    pos_Fig = get(hFigSAXSLee, 'position');
    pt = get(hAxes, 'CurrentPoint');
    xInd = (pt(1, 1));
    yInd = (pt(1, 2));
    xlim = get(hAxes, 'xlim');
    ylim = get(hAxes, 'ylim');
    if (xInd < min(xlim)) | (xInd > max(xlim))
        set(gcf,'Pointer','arrow');
        return
    end
    if (yInd < min(ylim)) | (yInd > max(ylim))
        set(gcf,'Pointer','arrow');
        return
    end
    set(gcf,'Pointer','crosshair');
    xp = 0.02;
    yp = 1;
    widthannotation = 300;%0.3;
    heightannotation = 10;%0.05;
    pR = '%3.5f'; % number of digit for position description.....
    posX = pos_hAxes(1) + xp*pos_hAxes(3);%posX = posX/pos_Fig(3);
%    posY = pos_hAxes(2) + (yp - heightannotation/2)*pos_hAxes(4);posY = posY/pos_Fig(4);
    posY = pos_hAxes(2) + yp*pos_hAxes(4) - heightannotation;%posY = posY/pos_Fig(4);
    hAnn = getappdata(hFigSAXSLee, 'Annohandle');
    if isempty(hAnn)
        hAnn = annotation('textbox',[0.1 0.1 0.1 0.1]);
        setappdata(hFigSAXSLee, 'Annohandle', hAnn);
    end
    
    set(hAnn, 'Units', 'pixel', 'position', [posX posY widthannotation heightannotation], 'linestyle', 'none');

% update figure title
    try
        pos_valuestr = ['X = ' num2str(xInd,pR) ', Y = ' num2str(yInd,pR) ', d = ' num2str(2*pi/xInd, '%0.1f')];
% possibility of wrong format strings...
    catch
        gtrack_Off()
    	error('GTRACK: Error printing coordinates. Check that you used a valid format string.')
    end
    set(hAnn, 'string', pos_valuestr);
    


function gtrack_Off(src,evnt)

% restore default figure properties
hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');

try
    hAnn = getappdata(hFigSAXSLee, 'Annohandle');
    setappdata(hFigSAXSLee, 'Annohandle', [])
    delete(hAnn);
end
fncs = getappdata(hFigSAXSLee, 'fncs');

set(hFigSAXSLee, 'windowbuttonmotionfcn', fncs.currFcn);
%set(hFigSAXSLee, 'windowbuttondownfcn', fncs.currFcn2);
set(hFigSAXSLee,'Pointer','arrow');
%title(handles.currTitle);



function gtrack_start(varargin)
%src,evnt
    % get current figure event functions
    hFigSAXSLee = findall(0,'Tag','SAXSLee_Fig');
    if strcmp(get(gcbo, 'State'), 'off')
        gtrack_Off;
        return
    end
    currFcn = get(hFigSAXSLee, 'windowbuttonmotionfcn');
%    currFcn2 = get(hFigSAXSLee, 'windowbuttondownfcn');
%    currTitle = get(get(gca, 'Title'), 'String');
% add data to figure handles
    %
    fncs.currFcn = currFcn;
%    fncs.currFcn2 = currFcn2;
    setappdata(hFigSAXSLee, 'fncs', fncs);
% set event functions 
    set(gcf,'Pointer','crosshair');
    set(gcf, 'windowbuttonmotionfcn', @gtrack_OnMouseMove);        
    %set(gcf, 'windowbuttondownfcn', @gtrack_OnMouseDown);          
