/*=================================================================
 * atm_jk.c 
 * Calculating F(q) of r_jk
 * Input should be F(atm) and atm
 * F(atm) is the 1*atm sized matrix
 * if atm = 1 and 4 were used, F(q, atm) should be 10*4 matrix...
 * Here, atm could be atomic number or number of a partice type starting from 1.
 * 
 * MEX-file.  NOTE: MATLAB uses 1 based indexing, C uses 0 based indexing.
 *
 * Takes a N-dimensional array of doubles and returns the indices for
 * the non-zero elements in the array. Findnz works differently than
 * the FIND command in MATLAB in that it returns all the indices in
 * one output variable, where the column element contains the index
 * for that dimension.
 *
 * This is a MEX-file for MATLAB.  
 * 2007. 12. 6.
 * Copyright 1984-2006 Byeongdu Lee.
 *============================================================*/

/* $Revision: 1.1 $ */

#include "mex.h"
#include <math.h>
#include <stdlib.h>

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    mwSize M;
    mwSize N;
    double *pr, *pind;
    long i, j;
    double *atm;
    int cnt;
    cnt = 0;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs != 2) {
	mexErrMsgTxt("Two input arguments required. f(q, atm) and atm");
    } 
    if (nlhs > 1){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
    M = mxGetNumberOfElements(prhs[0]);
    N = mxGetNumberOfElements(prhs[1]);
   
    /* Get the data */
    pr = mxGetPr(prhs[0]);
    atm = mxGetPr(prhs[1]);
            
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(N*(N-1)/2,1,mxREAL);
    pind=mxGetPr(plhs[0]);
	
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    for(i=0;i<N;i++) {
        for (j=i+1;j<N;j++) {
            if (((atm[i]-1) < M) & ((atm[j]-1) < M)) {
                pind[cnt] = pr[(int)(atm[i])-1]*pr[(int)(atm[j])-1];
                cnt++;
            }
            else {
                printf("i = %i, j = %i, atm(i) = %i, atm(j) = %i, M = %i\n", i, j, atm[i], atm[j], M);
                mexErrMsgTxt("number of atom is larger than that of Fq.");
            }
        }
    }
}
