#include "mex.h"
#include <math.h>

double sinc(double x)
{
if (x==0) return 1;
else return sin(x)/x;
}

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    mwSize atm_len;
    mwSize q_pnt;
    mwSize NE;
    double *pind;
    double *q, *atm_jk, *r_jk;
    double *jatm, *katm, *Ntype;
    double Nelement;
    long m, j;
    double calv;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs < 3) {
	mexErrMsgTxt("At least three input arguments required. q, atm_jk, r_jk or additionally j, k, Ntype");
    } 
    if (nlhs > 1){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
    /* Get the data */
    q = mxGetPr(prhs[0]);
    atm_jk = mxGetPr(prhs[1]);
    r_jk = mxGetPr(prhs[2]);
    atm_len = mxGetNumberOfElements(prhs[1]);
    q_pnt = mxGetNumberOfElements(prhs[0]);
    if (nrhs > 3) {
        jatm = mxGetPr(prhs[3]);
        katm = mxGetPr(prhs[4]);
        Nelement = mxGetScalar(prhs[5]);
    } else {
        Nelement = 0;
    }
    NE = (mwSize) Nelement + 1;
    
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
//    mexPrintf("n q is %" FMT_SIZE_T "\n", q_pnt);
//    mexPrintf("n element is %" FMT_SIZE_T "\n", NE);
    plhs[0]=mxCreateDoubleMatrix(q_pnt, NE, mxREAL);
    pind=mxGetPr(plhs[0]);
	
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    
    for (m=0;m<q_pnt;m++) {
        if (nrhs > 3) {
            for (j=0;j<NE;j++){
                pind[m + q_pnt*j] = 0.0;
            }
        } else {
            pind[m] = 0.0;
        }
        for (j=0;j<atm_len-1;j++) {
            calv = atm_jk[j]*sinc(q[m]*r_jk[j]);
//            pind[NE*m] += calv;
            pind[m] += calv;
            /* for partial structure factor*/
            if (nrhs > 3) {
                if (jatm[j] == katm[j]) {
//                    pind[NE*m + (int)jatm[j]] += calv;
                    pind[m + q_pnt*(int)jatm[j]] += calv;
                }
            }
        }
        if (nrhs > 3) {
            for (j=0;j<NE;j++){
                pind[m + q_pnt*j] = pind[m + q_pnt*j]*2.0;
            }
        } else {
            pind[m] = pind[m]*2.0;
        }
	}
}
