#include "mex.h"
#include <math.h>
//#include <time.h>

double sinc(double x)
{
if (x==0) return 1;
else return sin(x)/x;
}

double distance(double x1, double y1, double z1, double x2, double y2, double z2)
{
    double d;
    d = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2));
    return d;
}

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    mwSize atm_len;
    mwSize q_pnt;
    double *pind;
    double *q, *atm, *Fq, *pr;
//    double p;
    double r_jk;
    long m, j, i;
    double atm_jk;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs != 4) {
	mexErrMsgTxt("Four input arguments required. q, atm, coord, Fq");
    } 
    if (nlhs > 1){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
    // input: q, S, dl, ul, delta, w
    // output : q
//    elements=mxGetNumberOfElements(prhs[0]);
    /* Get the data */
    q = mxGetPr(prhs[0]);
    atm = mxGetPr(prhs[1]);
    pr = mxGetPr(prhs[2]);
    Fq = mxGetPr(prhs[3]);
//    numatm = mxGetNumberOfElements(prhs[1]);
    
    atm_len = mxGetNumberOfElements(prhs[1]);
    q_pnt = mxGetNumberOfElements(prhs[0]);
    
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(q_pnt,1,mxREAL);
    pind=mxGetPr(plhs[0]);
    
    
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    
    for (m=0;m<q_pnt;m++) {
        for (i=0;i<atm_len;i++) {
            for (j=i+1;j<atm_len;j++){
                r_jk = distance(pr[i], pr[i+atm_len], pr[i+2*atm_len],pr[j], pr[j+atm_len], pr[j+2*atm_len]);
                atm_jk = Fq[m + ((int)atm[i]-1)*q_pnt]*Fq[m + ((int)atm[j]-1)*q_pnt];
                pind[m] += atm_jk*sinc(q[m]*r_jk);
            }
        }
        pind[m] *= 2.0;
	}
}