function y = debyem(q, x, y, z, atm, Fq)
% y = debyem(q, x, y, z, atm, Fq)
% Debye slow calculator using exact formulae..
% Inputs:
%   coord : [x, y, z] and should have 3 column
%   atm : type of atm (1,2,3...) and should have the same number of element
%           of x
%   Fq : Fq(q, atmtype), thus numel(q) row and max(atmtype) column.
%        or Fq(1, atmtype), in this case Fq(q)_atom_i = i*ones(size(q));
% Outputs: I(q): numel(q) x (1+max(atm))
%   First column: total structure factor.
%   Second to the end: partial structure factor of atom 1 to end.
%
% Programmed by Byeongdu Lee.
% 2007. 12. 11
%
% debyefun2(q, a_jk, r_jk, atom j, atom k, Number of atoms);
%
% Other version:
% Method 1 by Ref. A. Cervellino, J. comput. Chem. 27: 995-1008, 2006.
%   y = debyem_fast(q, x, y, z, atm, Fq);
%   The equations are further modified by Byeongdu Lee.
% Method 2 using P(r)
%   y = debyepr(q, x, y, z, resolution, atm, Fq);
%   Use 500 for resolution.
%

q = q(:);
coord = [x(:), y(:), z(:)];

if nargin>4
    atmtype = unique(atm);
    t = numel(x)/numel(atm);
    if t > 1
        atm = repmat(atm, 1, t);
        atm = atm(:);
    end
else
    atmtype = 1;
end

if (numel(atmtype) == 1) && (nargin <= 5)
    atm = ones(size(x));
    y = debyefun(q, atm, x, y, z);
    return
else
    [nFq,n] = size(Fq);
    if n < max(atmtype)
        disp('Fq error. number of column of Fq should be bigger than the max value of atm')
        return
    end
    
    if nFq > 1 % when atomic scattering factor is q dependent.
        if numel(coord)/3 > 5000
            I0 = sum(abs(Fq(1:numel(q), atm)).^2, 2);
            y = I0 + debyefun3(q, atm(:), coord, Fq);
        else
            r_jk = distNzero(coord);
            y = zeros(size(q));
            for i=1:numel(q)
                a_jk = atm_jk(Fq(i, :), atm);
                I0 = sum(abs(Fq(i, atm)).^2);
                %y(i) = I0 + debyefun2(q(i), a_jk, r_jk)+1;
                y(i) = I0 + debyefun2(q(i), a_jk, r_jk);
            end
        end
    else  % when atomic scattering factor is q independent.
        %r_jk = distNzero(coord);
        [r_jk, ~, i, j] = distNzero(coord, atm);
        a_jk = atm_jk(Fq(1, :), atm);
        I0 = sum(abs(Fq(1, atm)).^2);
        %y(i) = I0 + debyefun2(q(i), a_jk, r_jk)+1;
        y = repmat(I0, length(q), 1) + debyefun2(q, a_jk, r_jk, atm(i), atm(j), max(atm(:)));
    end

end