function Iq = debyexy(qxy, qz, coordinate, atomtype, Fq)
% function Iq = debye2d(qxy, qz, coordinate, atomtype, Fq)
% The debye function for the 2D fiber symmetry.
% Derived by B. Lee
% 1/10/2013
% Two mode of calculation
%   qxy = [1,2,3,4,5] and qz = [1]
%   qxy = [1,2;3,4] and qz = [0,5;0,7]
% atomtype should be the same length with coordinate
% Fq should be (numel(q), Natomtype)
% if qxy and qz are 1D, they should be a column.
%
% example;
%   debyexy(qxy, qz, coordinate)
%   debyexy(qxy, qz, coordinate, atomtype, Fq)
sizeQxy = size(qxy);
if (numel(qxy) ~= 1) & (numel(qz) ~= 1)
        if numel(qxy) ~= numel(qz)
            error('size of qxy and qz should be the same')
        end
        %qxyzmode = 3; % mode 4 is fater than mode 3.
        qxyzmode = 4;
else
    if (numel(qxy) == 1)
        qxyzmode = 1;
    elseif  (numel(qz) == 1)
        qxyzmode = 2;
    end
end

q = sqrt(qxy.^2+qz.^2);
Np = length(coordinate);
if nargin < 4
    atomtype = ones(Np, 1);
end

Natomtype = numel(unique(atomtype));

if nargin < 5
    if size(q,2) == 1
        Fq = ones(numel(q), Natomtype);
    else
        Fq = ones(size(q,1), size(q,2), Natomtype);
    end
end
if size(q,2)>1
    Iinc = sum(Fq(:, :, atomtype).^2,3);
else
    Iinc = sum(Fq(:, atomtype).^2,2);
end
Ico = zeros(size(q));

if qxyzmode < 4
    for l=1:Np-1
        for m=l+1:Np
            %coordinate(l,:)
            %coordinate(m,:)
            rxy = sqrt((coordinate(l,1)-coordinate(m,1))^2+...
                (coordinate(l,2)-coordinate(m,2))^2);
            rz = abs(coordinate(l,3)-coordinate(m,3));
            %norm(coordinate(l,:)-coordinate(m,:))
            %sqrt(rxy^2+rz^2)
            switch qxyzmode
                case 1
                    Ico = Ico + Fq(:,atomtype(l)).*Fq(:,atomtype(m)).*cos(qz*rz)*sinc(qxy*rxy);
                case 2
                    Ico = Ico + Fq(:,atomtype(l)).*Fq(:,atomtype(m)).*sinc(qxy*rxy)*cos(qz*rz);
                case 3
                    if size(q,2)>1
                        Ico = Ico + Fq(:,:,atomtype(l)).*Fq(:,:,atomtype(m)).*sinc(qxy*rxy).*cos(qz*rz);
                    else
                        Ico = Ico + Fq(:,atomtype(l)).*Fq(:,atomtype(m)).*sinc(qxy*rxy).*cos(qz*rz);
                    end
            end
        end
    end
    Iq = Iinc + 2*Ico;
elseif qxyzmode == 4
        qxy = qxy(:);
        qz = qz(:);
        Fq = reshape(Fq, numel(qxy), Natomtype);
        Ico = debyexy_(qxy, qz, atomtype(:), coordinate, Fq);
        Iq = reshape(Ico, sizeQxy);
elseif qxyzmode == 5 % slower than mode 4, result the same.
        %qxy = qxy(:);
        %qz = qz(:);
        %Fq = reshape(Fq, numel(qxy), Natomtype);
        [rp, rz, atomk, atomj] = rkjpz_(atomtype, coordinate);
        %Ico = debyexy_(qxy, qz, atomtype(:), coordinate, Fq);
        for i=1:sizeQxy(1)
            Fq1 = reshape(Fq(i, :, atomk), sizeQxy(2), numel(atomk))';
            Fq2 = reshape(Fq(i, :, atomj), sizeQxy(2), numel(atomk))';
            Ico(i,:) = 2*sum(sinc((rp*qxy(i,:)).*cos(rz*qz(i,:))).*Fq1.*Fq2,1);
%            Ico(i,:) = 2*sum(sinc((rp*qxy(i,:)).*cos(rz*qz(i,:))),1);
        end
        Ico = reshape(Ico, sizeQxy);
        Iq = Iinc+Ico;
end