#include "mex.h"
#include <math.h>
#include <time.h>
#define MAXARRAY 100000

double sinc(double x)
{
if (x==0) return 1;
else return sin(x+0.000001)/(x+0.000001);
}

double distance(double x1, double y1, double z1, double x2, double y2, double z2)
{
    double d;
    d = sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2) + (z1-z2)*(z1-z2));
    return d;
}

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    size_t atm_len;
    size_t q_pnt;
//    size_t q_pnt;
    double *pind;
//    mxArray *p;
    double *atm, *pr;
//    double p;
    double *r_jkp, *r_jkz, *atomj, *atomk;
    long m, j, i;
    double atm_jk;
    double elapsed;
    long cnt;
    clock_t start, end;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs != 2) {
	mexErrMsgTxt("Two input arguments required. atm, coord");
    } 
    if (nlhs > 4){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
    // input: q, S, dl, ul, delta, w
    // output : q
//    elements=mxGetNumberOfElements(prhs[0]);
    /* Get the data */
    atm = mxGetPr(prhs[0]);
    pr = mxGetPr(prhs[1]);
//    numatm = mxGetNumberOfElements(prhs[1]);
    
    atm_len = mxGetNumberOfElements(prhs[0]);
//    q_pnt = mxGetNumberOfElements(prhs[0]);
    
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    //plhs[0]=mxCreateDoubleMatrix((mwSize)q_pnt,1,mxREAL);
    //pind=mxGetPr(plhs[0]);
    plhs[0]=mxCreateDoubleMatrix(atm_len*(atm_len-1)/2,1,mxREAL);
    r_jkp=mxGetPr(plhs[0]);
    plhs[1]=mxCreateDoubleMatrix(atm_len*(atm_len-1)/2,1,mxREAL);
    r_jkz=mxGetPr(plhs[1]);
    plhs[2]=mxCreateDoubleMatrix(atm_len*(atm_len-1)/2,1,mxREAL);
    atomk=mxGetPr(plhs[2]);
    plhs[3]=mxCreateDoubleMatrix(atm_len*(atm_len-1)/2,1,mxREAL);
    atomj=mxGetPr(plhs[3]);
    
    
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
//    start = clock();
    cnt = 0;
    for (i=0;i<atm_len;i++) {
        for (j=i+1;j<atm_len;j++){
            r_jkp[cnt] = distance(pr[i], pr[i+atm_len], 0,pr[j], pr[j+atm_len], 0);
            r_jkz[cnt] = distance(0, 0, pr[i+2*atm_len], 0, 0, pr[j+2*atm_len]);
            atomk[cnt] = atm[i];
            atomj[cnt] = atm[j];
            cnt++;
        }
    }
}