/*=================================================================
 * scatteringamp_.c 
 * Calculating Sq(qx,qy,qz) for (rx,ry,rz)
 * Input should be qx,qy,qz,rx,ry,rz
 * Return sum(cos(qx*rx+qy*ry+qz*rz)), the real term, for each (qx,qy,qz) and 
 * sum(sin(qx*rx+qy*ry+qz*rz)), the imag term, for each (qx,qy,qz)
 * 
 * MEX-file.  NOTE: MATLAB uses 1 based indexing, C uses 0 based indexing.
 *
 * Takes a N-dimensional array of doubles and returns the indices for
 * the non-zero elements in the array. Findnz works differently than
 * the FIND command in MATLAB in that it returns all the indices in
 * one output variable, where the column element contains the index
 * for that dimension.
 *
 * This is a MEX-file for MATLAB.  
 * 2013. 1. 6.
 * Copyright 1984-2006 Byeongdu Lee.
 *============================================================*/

/* $Revision: 1.1 $ */

#include "mex.h"
#include <math.h>
#include <stdlib.h>

void mexFunction(int nlhs,       mxArray *plhs[],
		 int nrhs, const mxArray *prhs[])
{
    /* Declare variables */ 
    mwSize M;
    mwSize N;
	double *qx, *qy, *qz, *rx, *ry, *rz;
    double *re, *im;
	double tempv;
    long i, j;
//    long cnt;
//    cnt = 0;
    
    /* Check for proper number of input and output arguments */    
    if (nrhs != 6) {
	mexErrMsgTxt("6 input arguments required. (qx,qy,qz),(rx,ry,rz)");
    } 
    if (nlhs > 2){
	mexErrMsgTxt("Too many output arguments.");
    }
    
    /* Check data type of input argument */
    if (!(mxIsDouble(prhs[0]))) {
      mexErrMsgTxt("Input array must be of type double.");
    }
    
    /* Get the number of elements in the input argument */
    M = mxGetNumberOfElements(prhs[0]);
    N = mxGetNumberOfElements(prhs[1]);
   
    /* Get the data */
    qx = mxGetPr(prhs[0]);
    qy = mxGetPr(prhs[1]);
    qz = mxGetPr(prhs[2]);
	//if (nrhs == 4) {
	//	R = mxGetPr(prhs[3]);
	//	} else {
    rx = mxGetPr(prhs[3]);
    ry = mxGetPr(prhs[4]);
	rz = mxGetPr(prhs[5]);
	//}
    /* Get the number of dimensions in the input argument. Allocate the
       space for the return argument */
    plhs[0]=mxCreateDoubleMatrix(M,1,mxREAL);
    plhs[1]=mxCreateDoubleMatrix(M,1,mxREAL);
    re=mxGetPr(plhs[0]);
    im=mxGetPr(plhs[1]);
	
    /* Fill in the indices to return to MATLAB. This loops through the
     * elements and checks for non-zero values. If it finds a non-zero
     * value, it then calculates the corresponding MATLAB indice and
     * assigns them into the output array.  The 1 is added to the
     * calculated indice because MATLAB is 1 based and C is zero
     * based. */
    for(i=0;i<M;i++) {
		re[i] = 0.0;
		im[i] = 0.0;
        for (j=0;j<N;j++) {
		    //printf("rx = %f, ry = %f, rz(i) = %f\n", R[i],R[i+1],R[i+2]);
			//if (nrhs == 4) {
			//tempv = q[i]*R[j]+q[i+1]*R[j+1]+q[i+2]*R[j+2];
			//} else {
			tempv = qx[i]*rx[j]+qy[i]*ry[j]+qz[i]*rz[j];
			//}
			re[i] = re[i] + cos(tempv);
			im[i] = im[i] - sin(tempv);
			//cnt++;
        }
    }
}
