%function imcalc(bmpfile)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function m-file for reading data directly
% from scanned pollen diagrams
% Version 2.0 8th June 1999
% Reads bitmap image - converts into binary matrix
% and calculates a series of data values
% NB (1) Written for stratigraphic data. (Y=independant axis)
%        (X=dependant axis)
%    (2) Requires a filled curve (see example data)
% Also requires a file (*.dep) with a series of depths
% to interpolate data. 
 
carryon=1;
while carryon==1
   
   [bmpfile, bmppath] = uigetfile('*.bmp', 'Open Scanned File:');
   
   if (bmppath == 0)
      break
      ;
   else
      
      % Read in file
      [imX,map]=imread([bmppath,bmpfile]);
      image(imX);
      axis image;
      colormap(map);
      % Enter limits: depths top and bottom and max percentage
      prompt={'Surface(CM):','Base(CM):','Max Percent:','Taxa No:','Linewidth:'};
      title='Curve Details';
      answer=inputdlg(prompt,title);
      mindep=str2num(char(answer(1,:))); % minimum depth values
      maxdep=str2num(char(answer(2,:))); % maximum depth value
      maxpc=str2num(char(answer(3,:))); % Maximum percentage value
      taxno=str2num(char(answer(4,:))); % Taxa number
      linew=str2num(char(answer(5,:))); % Line width (to remove)
      
      % find all rgb values greater than 0 (black)
      findwhite=find(imX>0);
      
      % replace all rgb values by 1 (make binary matrix)
      imX(findwhite)=1;
      
      % make vector of sums of 'black' (data values)
      sizX=size(imX,2);
      dataX=(sizX-(sum(imX,2)-linew));
      
      % rescale sums according to axis sizes
      dataX=dataX.*(maxpc/sizX);
      
      % make vector with Y/depth values
      sizY=size(imX,1);
      dataY=[1:sizY]';
      % rescale values using depths
      dataY=(dataY.*((maxdep-mindep)/sizY))+mindep;
      %i=find(Y<0);
      %Y(i)=0;
      
      % Load depth details
      %
      
      [depfile, deppath] = uigetfile('*.dep', 'Age-Depth: Open');
      if (deppath == 0)
         break
         ;
      else
         fdep=fopen([deppath,depfile],'r');
         [depdat]=(fscanf(fdep,'%f',[3 inf]))';
         [filename,count]=sscanf(depfile,'%c',((size(depfile,2))-4));
      end   
      
      % make output matrix
      taxout=(ones(size(depdat,1),1)).*taxno;
      % interpolate onto real depths
      outval=interp1q(dataY,dataX,depdat(:,3));
      outval=[depdat(:,1), depdat(:,2),taxout,outval];
      
      [outfile, outpath] = uiputfile([filename,'.out'], 'Output File');
      if (outpath == 0)
         ;
      else
         fout=fopen([outpath,outfile],'w');
         fprintf(fout,'%f %f %f %f \n',outval');
         fclose(fout);
      end
      
      k = menu('Do another', 'YES', 'NO');
      if k==1
         clear
         carryon=1;
      else
         carryon=2;
      end
      
      delete(gcf)
      fclose('all');
   end   
end
