function y = mvnpdf(x,mu,sigma)
% MVNPDF Multivariate normal density function.
%   Y = MVNPDF(X, MU, SIGMA) returns the multivariate normal 
%   density. X is a vector of values, MU is the 
%   mean parameter vector, and SIGMA is the covariance matrix. 

%   B.A. Jones 10-08-96
%   Copyright (c) 1993-96 by The MathWorks, Inc.

if size(x) ~= size(mu)
   error('The first two inputs must be vectors of the same length.')'
end

dim = max(size(x));
if size(sigma) ~= [dim dim]
   error('The third input must be a square matrix.');
end

if rank(sigma) < dim
   error('The third input must be a nonsingular matrix.');
end

if any(any(sigma ~= sigma'))
   error('The third input must be a symmetric matrix.');
end

if size(x,1) == 1,
   x = x(:);
   mu = mu(:);
end

R = chol(sigma);
det_s = prod(diag(R)).^2;
tmp = R'\(x-mu);
discrim = tmp'*tmp;
y = exp(-discrim/2)./sqrt(det_s.*(2*pi).^dim);
