function varargout = drawpolyhedron(varargin)
%drawpolyhedron Draw polyhedron defined by vertices and faces
%   h = drawpolyhedron(obj)
%   drawpolyhedron(obj, 'axis')
%   drawpolyhedron(vertices, faces)
%   h is a collection of handles for the patches.
%   Byeongdu Lee

%% Initialisations

Rotmat = [];

len_arrow = 0;
edgecolor = [];

if isstruct(varargin{1})
    obj = varargin{1};
    vertices = obj.vertices;
    faces = obj.faces;
    if isfield(obj, 'edgecolor')
        edgecolor = obj.edgecolor;
    else
        edgecolor = 'k';
    end
                
    if isfield(obj, 'facecolor')
        color = obj.facecolor;
    else
        color = '';
    end
    if isfield(obj, 'color')
        color = obj.color;
        if ischar(color)
            switch color
                case 'r'
                    color = [1, 0, 0];
                case 'b'
                    color = [0, 0, 1];
                case 'g'
                    color = [0, 1, 0];
                case 'm'
                    color = [1, 0, 1];
            end
        end
    end
    if isfield(obj, 'Rotmat')
        Rotmat = obj.Rotmat;
    end
    if numel(varargin) > 1
        arg = varargin{2};
        switch arg
            case 'axis'
                len_arrow = 1;
        end
    end
    alpha = [];
    if isfield(obj, 'facealpha')
        alpha = obj.facealpha;
    end
    %if ~isempty(color)
    %    varargin = [];
    varargin = [];
    varargin{1} = color;
    if ~isempty(alpha)
        varargin{2} = alpha;
    end
    
    %else
    %    varargin(1) = [];
    %end
else
    vertices = varargin{1};
    faces = varargin{2};
    if numel(varargin)>2
        color = varargin{3};
    else
        color = [];
    end
    if numel(varargin)>3
        alpha = varargin{4};
    else
        alpha = [];
    end
    varargin(1:2) = [];
end


% rotate the object if needed.
%if ~isempty(Rotmat)
%    vertices = vertices*Rotmat';
%end

% process input arguments
switch length(varargin)
    case 0 
        % default color is red
        %varargin = {'facecolor', [1 0 0]};
    case 1
        % use argument as color for faces
        if ~isempty(varargin{1})
            varargin = {'facecolor', varargin{1}};
        else
            varargin = [];
        end
    case 2
        % use argument as color for faces
        i1 = {'facecolor', varargin{1}};
        i2 = {'facealpha', varargin{2}};
        varargin = {i1{1:2}, i2{1:2}};
    case 3
        % use argument as color for faces
        i1 = {'facecolor', varargin{1}};
        i2 = {'facealpha', varargin{2}};
        len_arrow = varargin{3};
        varargin = {i1{1:2}, i2{1:2}};
    otherwise
        % otherwise do nothing
end

% overwrites on current figure
hold on;

% new addition
obj0.faces = faces;
obj0.vertices = vertices;
if ~isempty(edgecolor)
    obj0.edgecolor = edgecolor;
end

try    
    p = patch(obj0);
    %p = patch(obj);
    if ~isempty(color)
        p.FaceColor = color;
    else
        p.FaceColor = rand(3,1);
        p.FaceAlpha = 0.5;
    end
    if ~isempty(alpha)
        p.FaceAlpha = alpha;
    end
    if numel(faces)>1000
        p.EdgeColor = 'none';
        p.FaceAlpha = 0.5;
    end
catch
    if iscell(faces)
        Nf = numel(faces);
    else
        Nf = numel(faces(:,1));
    end
    for i=1:Nf
        %p(i) = patch('Faces',faces{i},'Vertices',vertices);
        if isempty(color)
            c = rand(1,3);
        else
            c = color;
        end
        if iscell(faces)
            ff = faces{i};
        else
            ff = faces(i, :);
        end
        p(i) = patch(vertices(ff, 1),vertices(ff, 2),vertices(ff, 3), c);
        
        %p = patch(obj);
        if ~isempty(color)
            p(i).FaceColor = color;
            if isempty(edgecolor)
                p(i).EdgeColor = 'none';
            else
                p(i).EdgeColor = edgecolor;
            end
        else
            p(i).FaceColor = rand(1,3);
            p(i).EdgeColor = [0, 0, 0];
        end
        if ~isempty(alpha)
            p(i).FaceAlpha = alpha;
        end
        if numel(faces)>1000
            p(i).EdgeColor = 'none';
            p(i).FaceAlpha = 0.5;
        end 
    end
end

if nargout > 0
    varargout = {p};
end

axis image;

if len_arrow
    dist = distancePoints3d(vertices(:,:), [0,0,0])*1.2;
    mdist = max(dist);
    mArrow3([0,0,0], [0,0,mdist*len_arrow], 'color','k');
    mArrow3([0,0,0], [0,mdist*len_arrow,0], 'color','b');
    mArrow3([0,0,0], [mdist*len_arrow,0,0], 'color','r');
end
% 
