function pts = intersect_planebox(varargin)
% boxplanes = intersect_planebox(Nvector)
%   box is given by axis range of gcf
%   p0 will be the center of the box
% boxplanes = intersect_planebox(Nvector, box)
%   p0 will be the center of the box
% boxplanes = intersect_planebox(Nvector, P0, box)

% The most part of the code is from drawPlane3d.m by David Legland.

if numel(varargin) >= 1
    Nvector = varargin{1};
    box = [xlim, ylim, zlim];
    P0 = [mean(xlim), mean(ylim), mean(zlim)];
end
if numel(varargin) == 2
    Nvector = varargin{1};
    box = varargin{2};
    P0 = [mean(box(1:2)), mean(box(3:4)), mean(box(5:6))];
end
if numel(varargin) == 3
    Nvector = varargin{1};
    P0 = varargin{2};
    box = varargin{3};
end

param = {'m'};
%if ~isempty(varargin)
%  param = varargin;
%end

% box = [xmin, xmax, ymin, ymax, zmin, zmax];
xmin = box(1);
xmax = box(2);
ymin = box(3);
ymax = box(4);
zmin = box(5);
zmax = box(6);

plane = createPlane(P0, Nvector);
% boxplanes = [0, 0, 0, 1, 0, 0, 0, 1, 0;
%     0,0,0, 1, 0, 0, 0, 0, 1;
%     0,0,0, 0, 1, 0, 0, 0, 1;
%     1, 1, 1, -1, 0, 0, 0, -1, 0;
%     1, 1, 1, -1, 0, 0, 0, 0, -1;
%     1, 1, 1, 0, -1, 0, 0, 0, -1];
% Cbox = [xmin, ymin, zmin];
% dbox = [xmax-xmin, ymax-ymin, zmax-zmin];
% boxmat = [dbox, dbox, dbox];
% boxplanes = boxplanes.*repmat(boxmat, size(boxplanes,1), 1);
% boxplanes(:,1:3) = boxplanes(:,1:3) + Cbox;
% 
% cen = Cbox + dbox/2;

% line corresponding to cube edges
lineX00 = [xmin ymin zmin 1 0 0];
lineX01 = [xmin ymin zmax 1 0 0];
lineX10 = [xmin ymax zmin 1 0 0];
lineX11 = [xmin ymax zmax 1 0 0];

lineY00 = [xmin ymin zmin 0 1 0];
lineY01 = [xmin ymin zmax 0 1 0];
lineY10 = [xmax ymin zmin 0 1 0];
lineY11 = [xmax ymin zmax 0 1 0];

lineZ00 = [xmin ymin zmin 0 0 1];
lineZ01 = [xmin ymax zmin 0 0 1];
lineZ10 = [xmax ymin zmin 0 0 1];
lineZ11 = [xmax ymax zmin 0 0 1];


% compute intersection points with each plane
piX00 = intersectLinePlane(lineX00, plane);
piX01 = intersectLinePlane(lineX01, plane);
piX10 = intersectLinePlane(lineX10, plane);
piX11 = intersectLinePlane(lineX11, plane);
piY00 = intersectLinePlane(lineY00, plane);
piY01 = intersectLinePlane(lineY01, plane);
piY10 = intersectLinePlane(lineY10, plane);
piY11 = intersectLinePlane(lineY11, plane);
piZ00 = intersectLinePlane(lineZ00, plane);
piZ01 = intersectLinePlane(lineZ01, plane);
piZ10 = intersectLinePlane(lineZ10, plane);
piZ11 = intersectLinePlane(lineZ11, plane);

% concatenate points into one array
points = [...
    piX00;piX01;piX10;piX11; ...
    piY00;piY01;piY10;piY11; ...
    piZ00;piZ01;piZ10;piZ11;];

% check validity: keep only points inside window
ac = 1e-14;
vx = points(:,1)>=xmin-ac & points(:,1)<=xmax+ac;
vy = points(:,2)>=ymin-ac & points(:,2)<=ymax+ac;
vz = points(:,3)>=zmin-ac & points(:,3)<=zmax+ac;
valid = vx & vy & vz;
pts = unique(points(valid, :), 'rows');

% If there is no intersection point, escape.
if size(pts, 1)<3
    disp('plane is outside the drawing window');
    return;
end

% the two spanning lines of the plane
d1 = plane(:, [1:3 4:6]);
d2 = plane(:, [1:3 7:9]);

% position of intersection points in plane coordinates
u1 = linePosition3d(pts, d1);
u2 = linePosition3d(pts, d2);

% reorder vertices in the correct order
ind = convhull(u1, u2);
ind = ind(1:end-1);