function obj = polyhedronshell(obj, shellthickness)
% This function make a shell around a polyhedron with a constant thickness.
% Algorithm: The code translates faces outward by the shellthickness in
% parallel and connects vertices. Thus, shell polyhedron is basically a
% truncated polyhedron of the input object.
%
% Byeongdu Lee
% Nov. 2017


% resolution
res = 10000;
cm = mean(obj.vertices);
obj.vertices = obj.vertices - repmat(cm, size(obj.vertices, 1), 1);

if iscell(obj.faces)
    Nfaces = numel(obj.faces);
else
    Nfaces = size(obj.faces, 1);
end
Va = [];
for i=1:Nfaces
    if iscell(obj.faces)
        F = obj.faces{i};
    else
        F = obj.faces(i, :);
    end
    [a, b, aeq, beq] = vert2lcon(obj.vertices(F, :));
    
    V = lcon2vert(a, b, aeq, beq+sign(beq)*shellthickness);
    Va = [Va;V];
end
t = unionHull('vert', Va);
Va = t.vert;
Va = unique(round(Va*res)/res, 'rows', 'stable');
try
    F = findfaces(round(Va*1000)/1000);
catch
    fprintf('Cannot produce faces for this object.\n');
    F = [];
end
Va = Va - repmat(cm, size(Va, 1), 1);
obj.vertices = Va;
obj.faces = F;
% t = drawpolyhedron(obj2);
% for i=1:numel(t)
%     t(i).FaceAlpha = 0.1;
% end



    