function obj = stellate(varargin)
% function obj = stellate(F, V, h)
% stellate(F, V, h) or stellate(obj, h)
% where h is the height of the stellated object.
%
if isstruct(varargin{1});
    obj = varargin{1};
    F = obj.faces;
    V = obj.vertices;
    h = varargin{2};
else
    F = varargin{1};
    V = varargin{2};
    h = varargin{3};
end

NF = length(F); NV = size(F,2);
Nvert = length(V);
Fa = [];
hold on; axis equal; view(3)
for i=1:NF;
    C(i,:) = sum(V(F(i,:),:))/NV;                   % center of a face
    H(i,:) = cross(C(i,:)-V(F(i,1),:), C(i,:)-V(F(i,2),:)); %normal
    if H(i,:)*C(i,:)'<0
        H(i,:) = -H(i,:);
    end
    NH = H(i,:)*H(i,:)';
    VH(i,:) = C(i,:) + h*H(i,:)/sqrt(NH);     % vertex
    Ftemp = [F(i,:), F(i,1)];
    for k=1:numel(Ftemp)-1;
        Fa = [Fa; Ftemp(k) Ftemp(k+1) Nvert+i];
    end
    for j=1:NV
        if j==NV; j1=1; else j1=j+1; end;
        W(i,j,:,:) = [VH(i,:); V(F(i,j),:); V(F(i,j1), :)]; % triangle
        patch(reshape(W(i,j,:,1), 1, 3), reshape(W(i,j,:,2), 1,3),...
            reshape(W(i,j,:,3), 1, 3), [i/NF/2, j/NV/2, 1-i/NF/2]);
    end
    stellate_verti = [V(F(i,:),:); VH(i,:)];
    stellate_faces = [];
    NstellateV = size(stellate_verti,1);
    Ftemp = [linspace(1,NstellateV-1,NstellateV-1), 1];
    for k=1:numel(Ftemp)-1
        stellate_faces = [stellate_faces;Ftemp(k), Ftemp(k+1),NstellateV] ;
    end
    stellate{i}.vertices = stellate_verti;
    stellate{i}.faces = stellate_faces;
end
obj.vertices = [V; VH];
obj.faces = Fa;
obj.stellate = stellate;
obj.stellateh = h;
obj.original.vertices = V;
obj.original.faces = F;
end

