function [b1, b2, b3] = LatticeRotate(b1, b2, b3, RotAxis, Rotangle, Dsp)
% Z = LatticeRotate(qx, qy, qz, b1, b2, b3, RotAxis, Rotangle, delA)
% This function is to rotate a fundamental or orthogonal vectors of Lattice
% around the given vector('RotAxis') in the sample coordinate(for example film normal
% direction is [0 0 1], x-ray direction is [1 0 0]) with amount of
% 'Rotangle' parameter.
%
% b1, b2, b3 should be determined for lattice orientation.
% Lattice determined by b1, b2, b3 can be rotated as 'Rotangle' about
% the axis of 'Rotaxis'.
% delA is the distortion factor.
%
% Assumption 
%  1. |a1| = |a2| = |a3| = a, here denoted as A.
%  2. isotropic distortion.

b1 = b1*Dsp ;
b2 = b2*Dsp ;
b3 = b3*Dsp;

b1 = eulerrot2(b1, RotAxis, Rotangle);
b2 = eulerrot2(b2, RotAxis, Rotangle);
b3 = eulerrot2(b3, RotAxis, Rotangle);