function Fq = diaz(varargin)
if numel(varargin) == 5
    qx = varargin{1};
    qy = varargin{2};
    qz = varargin{3};
    shape = varargin{4};
    L = varargin{5};
    a = polyhedra(shape, L);
    a = polyhedron_facecheck(a);
elseif numel(varargin) == 4
    % assuming polyhedron_facecheck is done.
    qx = varargin{1};
    qy = varargin{2};
    qz = varargin{3};
    a = varargin{4};
end

q = [qx(:), qy(:), qz(:)];
qn = vectorNorm3d(q)';
q = q';
% a = polyhedra(shape, L);
% a = polyhedron_facecheck(a);
faces = a.faces;
vertices = a.vertices;
Fq = zeros(1, numel(qx));
j = sqrt(-1);
Nfaces = numel(faces(:, 1));

for f_indx=1:Nfaces
    nqSk = surface_integral(f_indx, q, a);
    Fq = Fq + nqSk;
end

Fq = Fq(:)/j./qn(:).^2;

    function a = polyhedron_facecheck(a)
        faces = a.faces;
        vertices = a.vertices;
        Nfaces = numel(faces(:, 1));
        faceNorm = [];
        cm = [];
        ngb_faces = {};
        for i=1:Nfaces
            %ngb_face = [];
            f1 = faces(i, :);
            if iscell(f1)
                f1 = f1{1};
            end
            cm_k = mean(vertices(f1, :));
            fv = vertices(f1, :);
            n1 = planeNormal([fv(1, :), ...
                fv(2, :)-fv(1, :),...
                fv(3, :)-fv(1, :)]);
            n1 = n1/vectorNorm3d(n1);
            out = inPolyhedron2(a, cm_k + n1);
            %in = inPolyhedron2(a, cm_k - n1);
            %T = T/vectorNorm3d(T);
            if (out~=0) 
                faces(i,:) = fliplr(faces(i, :));
                n1 = -n1;
            end
            
            N_vertices_for_face1 = numel(f1);
            ngb_face = zeros(size(f1));
            for v_indx = 1:N_vertices_for_face1
                if v_indx < N_vertices_for_face1
                    v_pair = [v_indx, v_indx+1];
                else
                    v_pair = [v_indx, 1];
                end
                [~, f_indx2] = find_neighbor_face(i, f1(v_pair), faces);
                ngb_face(v_indx) = f_indx2;
            end
            ngb_faces{i} = ngb_face;
            faceNorm(i, :) = n1;
            cm(i,:) = cm_k;
        end
        a.faces = faces;
        a.ngb_faces = ngb_faces;
        a.faceNorm = faceNorm;
        a.cm = cm;
    end

    function [vn_NF, face_index] = find_neighbor_face(f1, vn, faces)
        % f1 : face number
        % vn : shared vertices, ex. [1,3]
        % vn_NF : vertices indices of the neighboring face. 
        N = 1:size(faces, 1);
        N(f1) = [];
        faces(f1, :) = [];
        if iscell(faces)
            ind = zeros(size(faces));
            for i=1:numel(faces)
                fc = faces{i};
                ind(i) = sum(ismember(fc(:)', vn), 2) == 2;
            end
        else
            ind = sum(ismember(faces, vn), 2) == 2;
        end
        if iscell(faces)
            nind = find(ind==1);
            vn_NF = faces{nind};
            face_index = N(nind);
        else
            vn_NF = faces(ind, :);
            face_index = N(ind);
        end
    end

%     function [ret, v_par] = isadjacentface(f1, f2)
%         v_par = intersect(f1, f2);
%         if numel(v_par)==2
%             ret = 1;
%         else
%             ret = 0;
%             v_par = 0;
%         end
%     end

    function nqSk = surface_integral(f_indx, q, a)
        f1 = a.faces(f_indx, :);
        if iscell(f1)
            if numel(f1) == 1
                f1 = f1{1};
            end
        end
        fv = a.vertices(f1, :);
        n1 = a.faceNorm(f_indx, :);
    
        area_k = facearea(fv);
        
        %Sk = zeros(1, size(q, 2));
        nq = n1*q;
        nq_nonzero = nq ~= 0;
        q_non0 = q(:, nq_nonzero);
        
        vt1 = fv(1, :);
        phasefactor = exp(j*vt1*q);
        Sk = area_k*phasefactor;
        
        if sum(nq_nonzero)>0  % for n_k * q is not 0,
            rep_n1 = repmat(n1', 1, size(q_non0, 2));
            qtan = cross(q_non0, rep_n1);
            qtan = vectorNorm3d(qtan')';
            k = qtan == 0; 
            Sk_line = zeros(1, size(q_non0, 2));
            
            % If qtan == 0, no further calculation required.
            % otherwise, line_integral is performed for edges.
            if sum(k) < numel(k)
                
                Sk_line(~k) = 0;
                % area integration becomes line-integration.
                %for f_indx2 = f_indx+1:Nfaces % finding faces around face1
                N_vertices_for_face1 = numel(f1);
                for v_indx = 1:N_vertices_for_face1
                    if v_indx < N_vertices_for_face1
                        v_pair = [v_indx, v_indx+1];
                    else
                        v_pair = [v_indx, 1];
                    end
                    % calculate
                    vt1 = fv(v_pair(1), :);
                    vt2 = fv(v_pair(2), :);


                    Cint = line_integral(n1, q_non0(:, ~k), qtan(~k), vt1, vt2);
                    %fprintf('face 1 is %i and face 2 is %i\n', f_indx, f_indx2)
                    Sk_line(~k) = Sk_line(~k) + Cint;
                end
                Sk(nq_nonzero) = Sk_line;
            end
        else
            fprintf('Face %i with n1 =[%0.3e %0.3e %0.3e] do not contribute\n', f_indx, n1)
        end
        nqSk = nq.*Sk;
    end

    function C_integral = line_integral(n1, q, qtan, vt1, vt2)
        % line integral and take into account the case of qT == 0.
        %T = cross(n1, n2);
        length_edge = vectorNorm3d(vt2-vt1);
        T = (vt2-vt1)/length_edge;
        
        phasefactor = exp(j*vt2*q);
        
        tripleScalarProd = T*cross(q, repmat(n1', 1, size(q, 2)));
        C_integral = j*length_edge*tripleScalarProd.*phasefactor;
        qT = T*q;
        m = qT == 0;
        if sum(m) < numel(m)
            C_integral(~m) = tripleScalarProd(~m)./qT(~m).*(exp(j*vt2*q(:, ~m))-exp(j*vt1*q(:, ~m)));
        end
        C_integral = C_integral./qtan.^2;
    end
    function area = facearea(vertices)
        if numel(vertices(:,1)) == 3
            area = triArea(vertices);
        else
            area = 0;
            for i=2:(numel(vertices(:,1))-1)
                vt = [vertices(1, :); vertices(i:i+1, :)];
                area = area + triArea(vt);
            end
        end
    end
    function a = triArea(vt)
        a = triangleArea3d(vt(1, :),vt(2, :),...
                vt(3, :));
    end
end