function [P, name, pnames, pin] = hollowCylinderForm3(q, p, flag)
% Formfactor of hollow cylinder3
% [P, name, pnames, pin] = hollowCylinderForm3(q, p, flag)
% currently, only fq are calculated, avFq2 = abs(fq).^2;
%    Rout = p(1);
%    Rin = p(2);
%    dR = p(3);
%    L = p(4);
%    rho1 = p(5);
%    rho2 = p(6);
%    I0 = p(7);
%    bkg = p(8);

% L. Ramos et al. Eur. Phys. J. B 1, 319(1998)
%n = length(rho);
%sumBottom = 0;
%for i=1:n-1
%    sumBottom = sumBottom + (rho(i) - rho(i+1))*R(i)^2;
%end

[xq, yq] = size(q);

if xq < yq 
    q = q';     % q should be column vector
end

if nargin == 2
    
    Rout = abs(p(1));
    Rin1 = abs(p(2));
    Rin2 = abs(p(3));
    dR = abs(p(4));
    L = abs(p(5));
    rho1 = abs(p(6));
    rho2 = abs(p(7));
    rho3 = abs(p(8));
    I0 = p(9);
    bkg = p(10);

    rho4 = 0;
    Rratio = Rin1/Rout;
    Hratio = L/Rout;
    
    if (dR ~= 0)
        [gDistR, Rout] = schultzdist99(Rout, dR, 10);gDistR=gDistR*(Rout(2)-Rout(1));
    end
    Lall = Hratio*Rout;
    Routall = Rout;Rin1all = Rratio*Rout;
    P = zeros(size(q));
    dalpha = pi/1000;
    for j = 1:length(Rout)
        R = [Rin2, Rin1all(j), Routall(j)];
        L = Lall(j);
        rho = [rho1, rho2, rho3, rho4];
        
        Y = zeros(size(q));
        for alpha = 0:dalpha:pi
            fq = 0;
            for i=1:length(R)
                fq = fq + 2*pi*L*(rho(i) - rho(i+1))*R(i)^2*besseljc(q*R(i)*sin(alpha)).*sinc(1/pi*q*L*cos(alpha)/2);%See matlab defination of sinc = sin(pi*x)/(pi*x)
            end
            Y = Y+abs(fq).^2*sin(alpha)*dalpha;
        end
        if (dR ~= 0)
            P = P + Y*gDistR(j);
        else
            P = P + Y;
        end
    end
    P = P/(pi*(p(1)^2-p(2)^2)*L)*I0 + bkg;
else
    
   P=[];
   name='Hollow Cylinder Model';
   
    pnames=str2mat('Rout', 'Rin1', 'Rin2', 'dR', 'L', 'rho1', 'rho2', 'rho3', 'I0', 'bkg');
	if flag==1, pin=[83, 50, 20, 0, 3000, 0, 1, 0.5, 1, 0]; else pin = p; end
end           