function [y, name, pnames, pin] = pedersen_2sp(x, p, flag)
% [y, name, pnames, pin] = pedersen(x, p, flag)
% Pedersen formalism to calculate scattering intensity.
% Local monodisperse approximation assumed.
% Formfactor assumed as sphere scattering.
% size distribution function : Gaussian(0), Gamma(1), lognormal(2),
% schultz(3)
% Structure factor : Hard sphere
%    center  = p(1);
%    width = p(2);
%    I0 = p(3);
%    bkg = p(4);
%    volfrac = p(5);
%    distrFunc = p(6);
%    dmax = p(7);
k = find(x == 0);
if ~isempty(k)
    if (k>0) & (k<length(x))
        x(k) = x(k+1);
    end
end


if nargin == 2;
    p1 = [p(1),p(2),p(3),p(4),p(5), 3, p(6), 0,0, -2, 0];
    p2 = [p(9),p(10),p(11),p(12),p(13), 3, p(14), 0,0,0,0];
    p3 = [p(1),p(2),0,0,0, 3, p(6), 0,0, p(7),p(8)];

    if p(2) > 0
        [gDist, xx] = gaussdist99(p(1), p(2));
    end

%    y1 = pedersen(x, p1);
% ===================== polydisperse lamellae with gaussian distribution
%    y1 = zeros(size(x));
%    for i=1:length(gDist)
%        y1 = y1 + gDist(i)*p(3)*abs(sin(x*xx(i))./(x*xx(1))).^2;
%    end
% ===================================================
%    y1 = cylindertype(x, [p(1), p(2), 10, 1, 1])*p(3);
    pcylinder = [116, p(1)-15, p(1)+15, 400, 600, p(2), -0.5, -0.6,p(4),0,0, 10000, 1];
    pcylinder = [116, p(1)-20, p(1)+20, 435, 500, 70, p(5), p(6), p(4), 0, 0, 10000, 1];
    pcylinder = [116, p(1), 350, 435, 500, 0, p(2), p(4), p(5), 0, 0, 0, 10000, 1];
    y1 = cylinderlayered(x, pcylinder)*p(3);
%    fl = ones(size(y1));
    fl = HexLattice2(x, 2*pi/0.0118, 0.16);
%    fl = HexLattice2(x, 2*pi/0.0128, 0.18);
%    fl = cylindertype3(x, 10000);
%    y1 = y1.*fl;
% ===================================================
    y2 = pedersen(x, p2);  % gold scattering
    y3 = pedersen(x, p3);  % power law curve...
%    y = y1.*y3 + y2;
    y = y1+ y2 + y3;
else
	y=[];
	name='Gauss distr Sphere';
	pnames=str2mat('xc1','w1', 'I01', 'eden1', 'eden2','eden3', 'power', 'prefactor', 'xc2','w2', 'I02', 'bkg2', 'Vol.f2', 'dmax2');
	if flag==1, pin=[450 45 10000 0, 0, 1, -2, 0, 12.5, 5.4, 1000, 0, 0.25, 2.2]; else pin = p; end
end

function SF = SFactor(q, Rhs, vf)
% vf is volume fraction of hard sphere
if vf == 0 
    SF = ones(size(q));
    return
end

alpha = (1 + 2*vf)^2/(1-vf)^4;
beta = -6*vf*(1 + vf/2)^2/(1-vf)^4;
gamma = vf*alpha/2;

if length(Rhs) > 1
    [R, q] = meshgrid(Rhs, q);
    A = R.*q*2;
else
    A = Rhs.*q*2;  
end

GA = alpha*(sin(A)-A.*cos(A))./A.^2 + ...
    beta*(2.*A.*sin(A) + (2 - A.^2).*cos(A)- 2)./A.^3 + ...
    gamma*(-1*A.^4.*cos(A) + 4*((3*A.^2-6).*cos(A) + (A.^3 - 6 * A).*sin(A) + 6))./A.^5;
SF = 1./(1+24*vf*GA./A);
%plot(SF(:,1))

function SF = SFactor2Dpara(q, dsp, sig)
% vf is volume fraction of hard sphere
F = exp(-1*q.^2*sig^2);
SF = (1-F.^2)./(1-2*F.*cos(q*dsp)+F.^2);
%plot(SF(:,1))