function [y, name, pnames, pin] = pedersen_AsymDGC(x, p, flag)
% [y, name, pnames, pin] = pedersen_AsymDGC(x, pDist, p)
% Pedersen formalism to calculate scattering intensity from a given distribution function.
% Asymmetric Double Gaussian Cumulative
% Local monodisperse approximation assumed.
% Formfactor assumed as sphere scattering.
% Structure factor : Hard sphere
% pDist = [x, Dist];
% p = [Rmin, Rmax, I0, center, width, shape1, shape2, volfrac, back];
if nargin == 2;
    
    Rmin = p(1);Rmax = p(2);
    xx = Rmin:(Rmax-Rmin)/40:Rmax;
    gDist = asymDGaussCum(xx, p(3:7));
    xx = xx';gDist = gDist';
    I0 = p(3);
    volfrac = abs(p(8)); % always positive
    back = p(9);
    [x1, x2]= size(x);
    
    if x1 < x2
        x = x';
    end
    %normFactor = sum(sum((gDist*xx.^6)));size(normFactor);%SS = SFactor(x, xx, volfrac);
    %y = sum(spheretype(x, xx).*SS*(gDist*xx.^6), 2)*I0/normFactor + bkg ;
    %normFactor = gDist'*(xx.^6);
    normFactor = sum(gDist);

    y = zeros(size(x));
%    figure;hold on
    for i = 1:length(xx)
        %Form = spheretype(x, xx(i))*xx(i)^6;  % let's discribe using volume distribution... because I don't want exaggerate small size particle...
        Form = spheretype(x, xx(i));
        Struc = SFactor(x, xx(i), volfrac);
        y = y + Form.*Struc.*gDist(i);
        %y = y + Form.*gDist(i);
%        plot(x, Form, 'b');plot(x, Struc, 'r');
    end
    y = y*I0/normFactor+back;

else
	y=[];
	name='Pedersen Asym';
   
    pnames = str2mat('Rmin','Rmax','I0', 'Center', 'Width', 'shape1', 'shape2','Volf','back');
	if flag==1, pin=[0, 100, 1, 50, 5, 5, 5, 0, 0]; else pin = p; end

end

function SF = SFactor(q, Rhs, vf)
% vf is volume fraction of hard sphere
alpha = (1 + 2*vf)^2/(1-vf)^4;
beta = -6*vf*(1 + vf/2)^2/(1-vf)^4;
gamma = vf*alpha/2;

if length(Rhs) > 1
    [R, q] = meshgrid(Rhs, q);
    A = R.*q*2;
else
    A = Rhs.*q*2;  
end

GA = alpha*(sin(A)-A.*cos(A))./A.^2 + ...
    beta*(2.*A.*sin(A) + (2 - A.^2).*cos(A)- 2)./A.^3 + ...
    gamma*(-1*A.^4.*cos(A) + 4*((3*A.^2-6).*cos(A) + (A.^3 - 6 * A).*sin(A) + 6))./A.^5;
SF = 1./(1+24*vf*GA./A+eps);
%plot(SF(:,1))