function [a, obj] = polyhedron_facecheck_situs(situs_filename)
% calculate facenormal vector and if it face inward, flip it to face out.
% required to use diazc.m
%
% Example usage;
% a = polyhedron_facecheck_situs('p3o58.situs')
% y2 = saxs_average(q, 'diazc', a);
% Generate the situs file from pdb file using 'pol2vol'
% or Use pdb2sax
% or y2 = debyepr(q, obj.x, obj.y, obj.z);
%
% 6/20/2015
% Byeongdu Lee

isolevel = 2;
maxfacenum = 500;
inoutcheckNpoint = 5;

% Reading situs file...
fprintf('Loading the situs file ''%s''...\n', situs_filename);
[t, x, y, z] = readsitus(situs_filename);
max_density = max(t(:));
fprintf('Generating isosurface with level/max %0.2f/%0.2f...\n', isolevel, max_density);
fv = isosurface(x, y, z, t, isolevel);
facenum = size(fv.faces, 1);

fprintf('Reduce number of patches...\n');
while (facenum > maxfacenum)
    fv = reducepatch(fv);
    facenum = size(fv.faces, 1);
end
k = t>=isolevel;
x = x(k);
y = y(k);
z = z(k);
a = fv;
obj.x = x;
obj.y = y;
obj.z = z;

    faces = a.faces;
    vertices = a.vertices;
    Nfaces = numel(faces(:, 1));
    faceNorm = [];
    cm = [];
%     ngb_faces = {};
fprintf('Checking face properties...\n');
    for i=1:Nfaces
        %ngb_face = [];
        f1 = faces(i, :);
        cm_k = mean(vertices(f1, :));
        fv = vertices(f1, :);
        n1 = planeNormal([fv(1, :), ...
            fv(2, :)-fv(1, :),...
            fv(3, :)-fv(1, :)]);
        n1 = n1/vectorNorm3d(n1);
        cm(i,:) = cm_k;
        d0 = sqrt((x-cm_k(1)).^2+(y-cm_k(2)).^2+(z-cm_k(3)).^2);
        [dv, di] = sort(d0);
        di = di(1:inoutcheckNpoint);
        dv = dv(1:inoutcheckNpoint);
        dv2 = sqrt((x-(cm_k(1)+n1(1))).^2+(y-(cm_k(2)+n1(2))).^2+(z-(cm_k(3)+n1(3))).^2);
        dv2 = dv2(di);
        if sum(dv2)<sum(dv)
            faces(i,:) = fliplr(faces(i, :));
            n1 = -n1;
fprintf('            Face %i is inverted.\n', i);
        end
        
        faceNorm(i, :) = n1;
%        out = inPolyhedron2(a, cm_k + n1);
        %in = inPolyhedron2(a, cm_k - n1);
        %T = T/vectorNorm3d(T);
%         if (out~=0) 
%             faces(i,:) = fliplr(faces(i, :));
%             n1 = -n1;
%         end

%         N_vertices_for_face1 = numel(f1);
%         ngb_face = zeros(size(f1));
%         for v_indx = 1:N_vertices_for_face1
%             if v_indx < N_vertices_for_face1
%                 v_pair = [v_indx, v_indx+1];
%             else
%                 v_pair = [v_indx, 1];
%             end
%             %[~, f_indx2] = find_neighbor_face(i, f1(v_pair), faces);
%             %ngb_face(v_indx) = f_indx2;
%         end
        %ngb_faces{i} = ngb_face;
    end
    a.faces = faces;
    %a.ngb_faces = ngb_faces;
    a.faceNorm = faceNorm;
    a.cm = cm;
fprintf('Done.\n');

end