function [Iq, r, pr, pv] = saxsPolyhedron(q, obj, forOrgn)
% function [Iq, r, pr, pv] = saxsPolyhedron(q, obj, forOrgn)
% This is develped for calculating Iq of any polyhedron
% for example.
% obj = polyhedra('icosahedron')
% and its stellated. 
% for instance
% obj = stellate(obj.faces, obj.vertices, 100*sqrt(2/3));
% However, when polyhedra is not a convex hull, it is not possible to
% correctly calculate whether a certain point is inside the polyhedron or
% not.
% Therefore, in order to calculate whether a point is in the stellated, 
% this software separately obtain points for each component of the stellated
% for instance, if the object is the stellated icosahedron,
% then 20 tetrahedra and 1 octahedron, each of them are convex hull..
% B. Lee, Nov 25 2012.

if nargin < 3
    forOrgn = 0;
end

CM = mean(obj.vertices);
nv = obj.vertices - repmat(CM, numel(obj.vertices(:,1)), 1);
minV = min(nv, [], 1);
maxV = max(nv, [], 1);
if isfield(obj, 'stellate')
    res = 60;
else
    res = 50;
end

x = linspace(minV(1), maxV(1), res) + CM(1);
y = linspace(minV(2), maxV(2), res) + CM(2);
z = linspace(minV(3), maxV(3), res) + CM(3);

dx = x(2) - x(1);
dy = y(2) - y(1);
dz = z(2) - z(1);
dv = dx*dy*dz;
%x = linspace(-mdist, mdist, 50);
[X, Y, Z] = ndgrid(x, y, z);
pv = [X(:), Y(:), Z(:)];
%pv = (rand(50^3, 3)-0.5)/0.5*mdist;

%inP = inpolyhedron(obj, pv);
if forOrgn == 1
    if isfield(obj, 'original');
        obj = obj.original;
    end
end
inP = inPolyhedron2(obj, pv);
k = inP==0; pv(k, :) = [];
Np = numel(pv(:,1));
V = dv*Np;
[Iq, r, pr] = debyepr(q, pv(:,1), pv(:,2), pv(:,3), 500);
Iq = Iq/Np^2*V^2;