function [y, V] = saxs_spiralcylinder2(qr, param)
% [FF, vol] = saxs_spiralcylinder2(qr, [Rh, R, ph, N])
% Calculate the form factor of a master curve of a helical cylinder.
% A helical cylinder is defined with Rh, R, L, N
% Rh : Radius of the helix in [nm or angstrom]
% R : Radius of the cylinder in [nm or angstrom]
% ph : pitch of the cylinder in [nm or angstrom]
% N : number of pitch
Rh = param(1);
R = param(2);
ph = param(3);
N = param(4);
    
v = ph/(2*pi*Rh);
vnz = (1+v^2)^(-1/2);
Rh_vnz = Rh/vnz;
Hcyl = 2*pi*Rh_vnz;

alpha = acos(2*pi*Rh./Hcyl);

Rg = N*Hcyl*sqrt(3)*cos(alpha)/(pi*sqrt(2))*exp(-1/2);
q1 = 1/Rg*sqrt(3/2);

y = zeros(size(qr));
rng = qr<q1;
y(rng) = (pi*R^2)^2*(N*Hcyl)^2*exp(-qr(rng).^2*Rg^2/3);
rng = qr>=q1;
y(rng) = (2*pi*R^2*N*Hcyl*besseljc(R*qr(rng))).^2./qr(rng)/(N*Hcyl)/cos(alpha)*pi;
V = pi*R^2*N*Hcyl;

