function FF = saxs_spiralspheres(qx, qy, qz, Rh, R, ph, N)
% FF = saxs_spiralcylinder(q, psi, chi, Rh, R, ph, N)
% Calculate the form factor of a helical cylinder.
% q, psi, and chi are m x 1 vector for q vectors.
% chi is the angle between the helic axis to the q vector.
% psi is the in-plane angle between q and the helix.
% A helical cylinder is defined with Rh, R, ph, N
% Rh : Radius of the helix in [nm or angstrom]
% R : Radius of the cylinder in [nm or angstrom]
% ph : pitch distance in [nm or angstrom]
% N : number of pitch

v = ph/(2*pi*Rh);
vnz = 1/sqrt(1+v^2);
circumference = pi*2*Rh;
helixlength = sqrt(ph^2+circumference^2);
Nh = fix(helixlength/(2*R)); % number of points to make a pitch.
t = linspace(0, 2*pi, fix(Nh)+1);t = t(:);
t(1) = [];
dt = t(2)-t(1);
j = sqrt(-1);
% for a given q, we need to integrate.
q = sqrt(qx.^2+qy.^2+qz.^2);
FF = zeros(size(q));
r = Rh*[cos(t), sin(t), v*t];
for i=1:numel(q)
    Fs = sphereamp(q(i), R);
    th = qx(i)*r(:,1)+qy(i)*r(:,2)+qz(i)*r(:,3);
    F = Fs.*sum(exp(-j*th));
    FF(i) = F.*(1-exp(-N*j*q(i)*ph))./(1-exp(-j*q(i)*ph));
end
end

