function [F, V] = saxsoctant(qx,qy,qz,RLH)
% octant with arbitary dimensions, R, L and H.
% Definition of this octant
% Bottom plane (isoless triangle)
%            ^ y   
%            |   
%            |   
%            | 
%   ----------------------->x
%          . | .
%         .  |  . R
%        .   |   .
%       .....|.....            
%            | L    
%
% H is the height of the octant.
% whose coordinate is at [0,0,H]
% 
% Coordinates of vertices
% vert = [0,0,0;
%       L/2, -sqrt(R^2-(L/2)^2), 0;
%       -L/2, -sqrt(R^2-(L/2)^2), 0;
%       0, 0, H]
% faces = [1,2,3;1,2,4;1,3,4;2,3,4]
%
% Byeongdu Lee
% Dec. 1. 2012

R = RLH(1);
L = RLH(2);
H = RLH(3);

qx = qx+eps;
qy = qy+eps*2;
%qz = qz+eps*20;

i = sqrt(-1);

tanth = L/sqrt((2*R)^2-L^2);
costh = sqrt(R^2-(L/2)^2)/R;
sinth = L/2/R;

C = [0,0,0; 0,0,H; R*sinth, -R*costh, 0; -R*sinth, -R*costh, 0];
V = 1/6*det([C(1,:)-C(2,:);C(1,:)-C(3,:);C(1,:)-C(4,:)]);



q1 = R/H*(costh*qy-sinth*qx)+qz;
q2 = R/H*(costh*qy+sinth*qx)+qz;

% sinc(qz*H/2);
Sz = sinc(qz*H/2);
%Sc = sinc(R*costh*qy+H/2*qz);
Ez = exp(-i*qz*H/2);
%Ez2 = exp(-i*qz*H);
%ER = exp(i*R*costh*qy);


D = qx.*(qy.^2-qx.^2*tanth^2)+eps;
F2 = qx.*Sz - ...
    1/2*Ez.*((qy/tanth+qx).*exp(i*q1*H/2).*sinc(q1*H/2) - ...
    (qy/tanth-qx).*exp(i*q2*H/2).*sinc(q2*H/2));
F = -2*H*tanth*Ez./D.*F2;

return

% The following does not work.
% I thought if I do reintegrate at singular point, for instance, for qx=0,
% I could calculate F at qx=0. But, the result was not.
% It seems that two functions are not continuous. I don't know why.
% but by avoiding the issue with adding different eps to q's,
% It seems all the singular conditions are avoided, resulting right answer.


% Now for specific cases.
n = abs(qx)<=eps;
ny = abs(qy)<=eps;

%%%%%%%
%n2 = abs(qy-qx*tanth)<=eps*2;
%n3 = n2 & ~n & ~ny;
%qy2 = qy(n3)+eps*2;
%D = qx(n3).*(qy2.^2-qx(n3).^2*tanth^2);
%F2 = qx(n3).*Sz(n3) - ...
%    1/2*Ez(n3).*((qy(n3)/tanth+qx(n3)).*exp(i*q1(n3)*H/2).*sinc(q1(n3)*H/2) - ...
%    (qy(n3)/tanth-qx(n3)).*exp(i*q2(n3)*H/2).*sinc(q2(n3)*H/2));
%F(n3) = -2*H*tanth*Ez(n3)./D.*F2;

%n2 = abs(qy+qx*tanth)<=eps*2;
%n3 = n2 & ~n & ~ny;
%qy2 = qy(n3)+eps*2;
%D = qx(n3).*(qy2.^2-qx(n3).^2*tanth^2);
%F2 = qx(n3).*Sz(n3) - ...
%    1/2*Ez(n3).*((qy(n3)/tanth+qx(n3)).*exp(i*q1(n3)*H/2).*sinc(q1(n3)*H/2) - ...
%    (qy(n3)/tanth-qx(n3)).*exp(i*q2(n3)*H/2).*sinc(q2(n3)*H/2));
%F(n3) = -2*H*tanth*Ez(n3)./D.*F2;


q1 = qy*R/H*costh+qz;

% for qy-tanth*qx == 0
n2 = abs(qy-qx*tanth)<=eps*2;
n3 = n2 & ~n & ~ny;
%qy2 = qy(n3)+eps;
q1_qz = q1(n3)-qz(n3);
T2 = H./qx(n3)./qy(n3).*Ez(n3);
F(n3) = T2.*((qy(n3)*R*costh - i/2 + i*R/H*costh).*Sz(n3) -...
    i*R/H*costh*Ez(n3)+i/2*exp(i*q1_qz*H/2).*sinc(q1(n3)*H/2));
%F(n3) = 0;
% for qy+tanth*qx == 0
n2 = abs(qy+qx*tanth)<=eps*2;
n3 = n2 & ~n & ~ny;
q1_qz = q1(n3)-qz(n3);
T2 = H./qx(n3)./qy(n3).*Ez(n3);
F(n3) = T2.*((qy(n3)*R*costh + i/2 + i*R/H*costh).*Sz(n3) -...
    i*R/H*costh*Ez(n3)-i/2*exp(i*q1_qz*H/2).*sinc(q1(n3)*H/2));
%F(n3) = 0;






% for qx == 0
q1_qz = q1(n)-qz(n);
F(n) = H*tanth./qy(n).^2.*(exp(i*(q1(n)/2-qz(n))*H).*sinc(q1(n)*H/2).*(1-...
    i*H*q1_qz+q1_qz./q1(n))-Ez(n).*Sz(n)-q1_qz./q1(n).*Ez2(n));

% for qx ==0 & q1 ==0
nqn = abs(q1)<=eps;
nqxqn = n&nqn;
F(nqxqn) = tanth./qy(nqxqn).^2*H.*((1+i*H*qz(nqxqn)...
    -i*H^2*qz(nqxqn).*qy(nqxqn)/2).*Ez2(nqxqn) -...
    Ez(nqxqn).*Sz(nqxqn));


% for qx==0 & qy==0;
nqxqy = n&ny;
F(nqxqy) = tanth*costh^2*R^2*((H-2*i./qz(nqxqy)-...
    2/H./qz(nqxqy).^2).*Ez(nqxqy).*Sz(nqxqy) + ...
    (2/H./qz(nqxqy).^2-i./qz(nqxqy)).*Ez2(nqxqy));
% for qx==0 & qy==0 & qz==0
nz = abs(qz)<=0;
nqxqyqz = n&ny&nz;

F(nqxqyqz) = V;


end

