function [F, Ftet, Foct] = saxsstellateoctahedron3(qx, qy, qz, RH)
% calculate F of saxsstellateoctahedron by tranforming coordinates 
% instead of transforming particles.
% but, requires model for tetrahedron.
%   tet = polyhedra('tetrahedron'); tet.vertices = tet.vertices*R;
%

if numel(RH) == 1
    R = RH;
    H = sqrt(2/3)*L;    % Lengthes of all edges of this tetrahedron is the same.
else
    R = RH(1);H = RH(2);
end

Foct = saxsoctahedron(qx,qy,qz,R);

tet = evalin('base', 'tet');

sinth = sqrt(2/3);
ang = asin(sinth);
angRot = [ang, ang+pi];
%angRot = [ang];
angIn = pi/2*[0, 1, 2, 3];
if numel(size(qx))==2
    Ftet = zeros(size(qx,1), size(qx,2), 8);
else
    Ftet = zeros(size(qx));
end

j = sqrt(-1);
Qall = [qx(:), qy(:), qz(:)];
for l=1:numel(angRot)
    ang = angRot(l);
    for m=1:4
        angxy = angIn(m);
        
        MRP = [1,0,0;0,cos(ang), -sin(ang);0, sin(ang), cos(ang)]; % for Rotation for Particle.
        MRPxy = [cos(angxy), -sin(angxy) 0;sin(angxy), cos(angxy) 0; 0,0,1];

        Q = (inv(MRPxy*MRP)*Qall')';
        q1 = reshape(Q(:,1), size(qx));
        q2 = reshape(Q(:,2), size(qx));
        q3 = reshape(Q(:,3), size(qx));
        Zt = R/sqrt(3)*sin(ang);
        Yt = -(1-cos(ang)/sqrt(3))*R;
        %fprintf('ang, angxy, Zt and Yt = %0.3f, %0.3f, %0.3f and %0.3f\n',ang, angxy, Zt,Yt)
        switch angxy
            case 0
                Zm = exp(-j*qz*Zt);
                Ym = exp(-j*qy*(-1*abs(Yt)));
            case pi/2
                Zm = exp(-j*qz*Zt);
                Ym = exp(-j*qx*abs(Yt));
            case pi
                Zm = exp(-j*qz*Zt);
                Ym = exp(-j*qy*abs(Yt));
            case pi/2*3
                Zm = exp(-j*qz*Zt);
                Ym = exp(-j*qx*(-1*abs(Yt)));
        end
        
        tt = 4*(l-1) + m;
        Vtet = tet.volume; % tet should contain its volume...
        %Vtet = 1.1785e+05; % for R = 50;
        F0 = saxsPolyhedronAmp(q1,q2,q3,tet)*Vtet;
        if numel(size(q1)) == 2
            Ftet(:,:,tt) = F0.*Zm.*Ym;
        else
            Ftet = Ftet + F0.*Zm.*Ym;
        end

        %drawpolyhedron(temp.vertices, temp.faces, 'r')
    end
end

%Vtet = 1.1785e+05;
%Ftet = Ftet*Vtet;
if numel(size(Ftet))==2
    F = Foct + sum(Ftet, 3);
else
    F = Foct + Ftet;
end