function [F, V, S] = saxstetrahedron2(qx, qy, qz, L)
% function [F, V, S] = saxstetrahedron2(qx, qy, qz, L)
% Coded based on G. Renaud's paper; Surf. Sci. Rep. 2009, 64, 255.
% Note that there was typo and fixed in this code.
if numel(L) == 2;
    H = L(2);
    L = L(1);
else
    H = sqrt(2/3)*L;    % Lengthes of all edges of this tetrahedron is the same.
end

R = L/2;
tanalp = H/R*sqrt(3);
qx = -qx;
qy = -qy;
qz = -qz;
V = 1/3*tanalp*(R^3-(R-sqrt(3)*H/tanalp)^3);
S = 4*R*sqrt(H^2+R^2/3);

i = sqrt(-1);
q1 = 1/2*((sqrt(3)*qx-qy)/tanalp - qz);
q2 = 1/2*((sqrt(3)*qx+qy)/tanalp + qz);
q3 = 1/2*(2*qy/tanalp - qz);

L = 2*tanalp*R/sqrt(3) - H;
D = (sqrt(3)*qx.*(qx.^2-3*qy.^2))+eps;
D(abs(D)<eps) = eps;

F = H./D.*exp(i*qz*R*tanalp/sqrt(3));
T = (-(qx+sqrt(3)*qy).*sinc(q1*H).*exp(i*q1*L) + ...
    (-qx+sqrt(3)*qy).*sinc(q2*H).*exp(-i*q2*L) + 2*qx.*sinc(q3*H).*exp(i*q3*L));
F = F.*T;
F = F*3;% added to correct typo in the G. Renaud's paper; Surf. Sci. Rep. 2009, 64, 255.