function F = saxstriangle3(qx0, qy0, R, angle)
%F = saxstriangle(qx,qy,R,angle)
% analytical form factor of a isosceles triangle
% R is the length of the perpendicular line between the acute and the bottom.
% A is at [0,0] and R = AC and AB.
%            . A
%           .  .
%          .    .
%         .       .
%        .        .
%      B. . . . . . .C
%
%
% bottom of the triangle is parallel to x axis
% its center of mass is on negative side of y axis.
% derived by Byeongdu Lee. 2012, 12, 1.
% 
% Integration for singularities is redone. 2018. 12. 2.
% 

j = sqrt(-1);

tol = 1E-15;

sinth = sin(angle);
costh = cos(angle);
tanth = tan(angle);

B1 = (qy0.^2-tan(angle)^2*qx0.^2);
B = qx0.*B1+eps;

con1 = abs(qx0) < tol;
con2 = abs(qy0) < tol;
con3 = abs(B1) < tol;

t{2} = con1 & con2;
t{3} = ~con1 & con3;
t{4} = con1 & ~con2;
t{1} = ~(t{2} | t{3} | t{4});

F = zeros(size(qx0));

for i=1:4
    qx = qx0(t{i});
    qy = qy0(t{i});
    Bn = B(t{i});
    switch i
        case 1
            F0 = -2*tanth./Bn.*exp(j*R*costh*qy);
            F(t{i}) = F0.*(qx.*exp(-j*costh*R*qy)-qx.*cos(qx*R*sinth)+j/tanth*qy.*sin(qx*R*sinth));
        case 2 % qx=0 and qy=0
            F(t{i}) = R^2*costh^2*tanth;
        case 4 % qx=0 and qy~=0
            F(t{i}) = -2*tanth./qy.^2.*(1+j*exp(j*qy*R*costh).*(qy*R*costh+j));
        case 3 % qx~=0 and qy = tan(th)*qx;
            %F(t{i}) = 2./qx.^2/tanth.*(1-exp(j*qx*R*tanth*costh))+2*j*R*costh./qx;
            F(t{i}) = 1./qx.^2/tanth.*(1/2-1/2*exp(2*j*qx*R*sinth)+j*qx*R*sinth);
    end
end
