function y = spheroid_vert_clusters_pDist(x, pDist, p, strfactor, strp, zdep)
% [y, name, pnames, pin] = pedersen_pDist(x, pDist, p)
% Pedersen formalism to calculate scattering intensity from a given distribution function.
% Local monodisperse approximation assumed.
% Formfactor assumed as sphere scattering.
% Structure factor : Hard sphere
% pDist = [x, Dist];
% p = [I0, volfrac, back];
%    global kkk

if nargin > 4;
    xx = pDist(:, 1);
    gDist = pDist(:, 2);
    if isfield(p, 'hx')
        xh = p.hx;
        hDist = p.nh;
    else
        xh = [];
        hDist = [];
    end
        if isfield(x, 'q')
            qpa = x.qpa;
            q1z = x.q1z;
            q2z = x.q2z;
            q3z = x.q3z;
            q4z = x.q4z;
            Ti = x.Ti;
            Tf = x.Tf;
            Ri = x.Ri;
            Rf = x.Rf;
            GI = 1;
        else
            x = x(:);
            qpa = x;
            q1z = 0;
            q2z = 0;
            q3z = 0;
            q4z = 0;
            Ti = 1;
            Tf = 0;
            Ri = 0;
            Rf = 0;
            GI = 0;
        end
  I0 = p.I0;
  
  if isempty(xh)
    RcRp = p.RcRp;
    RcbRc = p.RcbRc;
    normFactor = sum(gDist.*(xx.^6))*RcRp^2*(4/3*pi)^2;  % for cylinder
    y = zeros(size(qpa));
    for i = 1:length(xx)
        F = dwbasph(qpa, q1z, q2z, q3z, q4z, Ti, Tf, Ri, Rf, xx(i), RcRp, RcbRc, [1, 0], zdep);%xx(i)*RcbRc*RcRp);
        if ~isempty(strp)
            Struc = feval(strfactor, x, strp, xx(i));  % for strfactor2
        else
            Struc = 1
        end
        y = y + abs(F).^2.*Struc*gDist(i);
    end
    y = y*p.I0/normFactor;
  else
    RcbRc = p.RcbRc;
%    R = xx;
%    H = xh;
%    gR = gDist;
%    gH = hDist;
    [R, H] = meshgrid(xx, xh);
    [gR, gH] = meshgrid(gDist, hDist);
    R = R(:); H = H(:); gR = gR(:); gH = gH(:);
    RcRp = 1/(p.RcbRc+1)*H./R;
    normFactor = sum(gR.*gH.*(R.^6).*RcRp.^2)*(4/3*pi)^2;  % for sphere
    y = zeros(size(qpa));
%    kkk = [R, H, gR, gH, RcRp];
    for i = 1:length(R)
        F = dwbasph(qpa, q1z, q2z, q3z, q4z, Ti, Tf, Ri, Rf, [R(i)*p.Rshell, R(i)], RcRp(i), RcbRc, [p.rhoShell, 1, 0], zdep);%xx(i)*RcbRc*RcRp);
        if ~isempty(strp)
            Struc = feval(strfactor, x, strp, R(i));  % for strfactor2
        else
            Struc = 1;
        end
        y = y + abs(F).^2.*Struc*gR(i)*gH(i);
    end
    y = y*p.I0/normFactor;
  end    
end