#include "math.h"
#include "mex.h"

/* The gateway routine */
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
    mxArray *xD, *yD, *xxD;
    double *xa, *ya, *qp, *qz1, *qz2;
    double *y, sig1, sig2, zint1, zint2, hcor, vcor, H;
    double yt[1];
    double q1, q2, q3;
    int nn;
    int mrows, ncols;
    int mxrows, nxcols;
    int ind,m;

  if (nrhs != 12) 
    mexErrMsgTxt("Twelve inputs required.");
  if (nlhs != 1) 
    mexErrMsgTxt("One output required.");
    xD = prhs[0];
    yD = prhs[1];

    xxD = prhs[2];
    xa = mxGetPr(xD);
    ya = mxGetPr(yD);
    qp = mxGetPr(xxD);
    qz1 = mxGetPr(prhs[3]);
    qz2 = mxGetPr(prhs[4]);
    sig1 = mxGetScalar(prhs[5]);
    sig2 = mxGetScalar(prhs[6]);
    zint1 = mxGetScalar(prhs[7]);
    zint2 = mxGetScalar(prhs[8]);
    hcor = mxGetScalar(prhs[9]);
    vcor = mxGetScalar(prhs[10]);
    H = mxGetScalar(prhs[11]);
    
    mxrows = mxGetM(xxD);
    nxcols = mxGetN(xxD);
    mrows = mxGetM(xD);
    ncols = mxGetN(xD);
    nn = mrows;

    /* Output setting */
    plhs[0] = mxCreateDoubleMatrix(mxrows, nxcols, mxREAL);
    y = mxGetPr(plhs[0]);

    if (ncols > mrows) nn = ncols;
    ind = 1./xa[1];
   
    for (m=0;m<(mxrows*nxcols);m++) {
//        printf("%f, %f, %f, %f, %f, zint1 = %f, %f, %f, %f, %f\n", qp[m],qz1[m],qz2[m],sig1,sig2,zint1,zint2,hcor,vcor,H);
        q1 = qp[m];
        q2 = qz1[m];
        q3 = qz2[m];
        corr(xa, ya, nn, ind, q1,q2,q3, sig1, sig2, zint1, zint2, hcor, vcor, H, yt);
        y[m] = yt[0];
        }
}

double polint(double *xa, double *ya, int n, int indx, double x, double *y)
    /*n is the total array size. indx is the index/x ratio*/
{
    float y1, y2;
    int x1, x2;
    float indxx, dy, dx, dxx;
    int ind;
    if (x<0) mexErrMsgTxt("x should be not less than 0.");
    if (x>xa[n-1]) mexErrMsgTxt("x is larger than max limit of xa\n");
    indxx = x*indx;
    ind = floor(indxx);
    x1 = floor(indxx);
    x2 = floor(indxx)+1;
    
    if (abs(xa[x1]-x)*indx > xa[1]) {
        *y = -99;
        mexErrMsgTxt("xa is not increasing with a slope of 1");
        }
    if (x2<x1) {
        *y=ya[x2];
        mexErrMsgTxt("x2 is larger than x1, round off error of C");
        }
    y1 = (float) ya[x1];
    y2 = (float) ya[x2];
    
    if ((x>xa[x1]) & (x<xa[x2])) {
        dy = y2-y1;
        dx = x2-x1;
        dxx = x-xa[x1];
        *y = dy/dx*dxx + y1;
        }
    if (x<=xa[x1]) *y = y1;
    if (x>=xa[x2]) *y = y2;
    return 0;
}


void corr(double *xa, double *ya, int nn, int ind, double qx, double qz1, double qz2, double sig1, double sig2, double zint1, double zint2, double hcor, double vcor, double H, double *y)
{
    int i;
    float r;
    float rmax, r_stp;
    float numpnt=20.0;
    float numsm = 2000.0;
    double corr_end=0.001;
    double Cr;
    double yt[1];
	double ratio;
    rmax = hcor*exp(log(log(1/corr_end))/2/H);
    r_stp = 2.0*3.141592/(qx+0.0000001)/numpnt;
    ratio = rmax/r_stp;
    if (ratio < 10) r_stp = rmax/numsm;
    Cr = 0.0;
//    printf("r_stp = %f, rmax = %f\n", r_stp, rmax);
    for (r=0;r<=rmax;r=r + r_stp) {
            polint(xa, ya, nn, ind, qx*r, yt);
            Cr = Cr + (exp((qz1*qz2)*sig1*sig2*exp(-1*pow((r/hcor),(2*H))-abs(zint1-zint2)/vcor))-1)*r*yt[0];
        }
    y[0] = Cr*r_stp;
}


